﻿using Autodesk.Forge;
using Autodesk.Forge.Model;
using Microsoft.AspNetCore.Mvc;
using Microsoft.AspNetCore.Http;
using Newtonsoft.Json.Linq;
using RestSharp;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Net;
using System.Threading.Tasks;
namespace TrabajoBIM360
{
    public class GestionIssuesController : Controller
    {/// <summary>
     /// Credentials on this request
     /// </summary>
        private Credentials Credentials { get; set; }

        [HttpGet]
        [Route("api/forge/issues/container")]
        public async Task<string> GetContainerAsync(string hubId, string projectId)
        {
            Credentials = await Credentials.FromSessionAsync(base.Request.Cookies, Response.Cookies);
            if (Credentials == null) { return null; }
            ProjectsApi projectsApi = new ProjectsApi();
            projectsApi.Configuration.AccessToken = Credentials.TokenInternal;
            var project = await projectsApi.GetProjectAsync(hubId, projectId);
            var issues = project.data.relationships.issues.data;
            if (issues.type != "issueContainerId") return null;
            return issues["id"];
        }


        [HttpGet]
        [Route("api/forge/issues/listado")]
        public async Task<JArray> GetDocumentIssuesAsync(string containerId, string hubId, string urn)
        {

            System.Net.Http.HttpClient httpClient = new System.Net.Http.HttpClient();
            Microsoft.AspNetCore.Http.Headers.RequestHeaders header = this.Request.GetTypedHeaders();
            Uri uriReferer = header.Referer;

            httpClient.BaseAddress = new Uri(uriReferer.AbsoluteUri);
            dynamic responseUsers = await httpClient.GetStringAsync("/api/forge/user/showAllUsers?hubId=" + hubId);
            JArray users = JArray.Parse(responseUsers);

            const string BASE_URL = "https://developer.api.autodesk.com";
            Credentials = await Credentials.FromSessionAsync(base.Request.Cookies, Response.Cookies);
            if (Credentials == null) { return null; }

            RestClient client = new RestClient(BASE_URL);
            RestRequest request = new RestRequest("/issues/v1/containers/{container_id}/quality-issues", RestSharp.Method.GET);
            request.AddParameter("container_id", containerId, ParameterType.UrlSegment);
            request.AddQueryParameter("filter[target_urn]", urn);
            request.AddHeader("Authorization", "Bearer " + Credentials.TokenInternal);
            IRestResponse documentIssuesResponse = await client.ExecuteAsync(request);
            dynamic issues = JObject.Parse(documentIssuesResponse.Content);

            foreach (dynamic issue in issues.data)
            {
                issue.attributes.assigned_to_name = "Not yet assigned";
                foreach (dynamic user in users)
                {
                    if (user.uid == issue.attributes.assigned_to)
                    {
                        issue.attributes.assigned_to_name = user.name;
                    }
                    //Repetir para cada uno de los atributos cun "uid"
                }
            }

            return issues.data;
        }

        [HttpPost]
        [Route("api/forge/issues/{containerId}/issues/{urn}")]
        public async Task CreateDocumentIssuesAsync(string containerId, string urn, [FromBody] JObject data)
        {
            // for this sample, let's create Design issues
            // so we need the ngType and ngSubtype
            IRestResponse issueTypesResponse = await GetIssueTypesAsync(containerId);
            dynamic issueTypes = JObject.Parse(issueTypesResponse.Content);
            string ngTypeId = string.Empty;
            string ngSubtypeId = string.Empty;
            foreach (dynamic ngType in issueTypes.results)
            {
                if (ngType.title == "Design") // ngType we're looking for
                {
                    foreach (dynamic subType in ngType.subtypes)
                    {
                        if (subType.title == "Design") // ngSubtype we're looking for
                        {
                            ngSubtypeId = subType.id; break; // stop looping subtype...
                        }
                    }
                    ngTypeId = ngType.id; break; // stop looping type...
                }
            }
            // double check we got it
            if (string.IsNullOrWhiteSpace(ngTypeId) || string.IsNullOrWhiteSpace(ngSubtypeId))
                return;// BadRequest();
                       // and replace on the payload
            data["data"]["attributes"]["ng_issue_type_id"] = ngTypeId;
            data["data"]["attributes"]["ng_issue_subtype_id"] = ngSubtypeId;
            // now post to Quality-Issues
            await PostIssuesAsync(containerId, data);
        }

        public async Task<IRestResponse> GetIssueTypesAsync(string containerId)
        {
            System.Net.Http.HttpClient httpClient = new System.Net.Http.HttpClient();
            Microsoft.AspNetCore.Http.Headers.RequestHeaders header =
           this.Request.GetTypedHeaders();
            Uri uriReferer = header.Referer;
            httpClient.BaseAddress = new Uri(uriReferer.AbsoluteUri);
            const string BASE_URL = "https://developer.api.autodesk.com";
            Credentials = await Credentials.FromSessionAsync(base.Request.Cookies,
           Response.Cookies);
            if (Credentials == null) { return null; }
            RestClient client = new RestClient(BASE_URL);
            RestRequest request = new RestRequest(
            "/issues/v1/containers/{container_id}/ng-issue-types?include=subtypes",
           RestSharp.Method.GET);
            request.AddParameter("container_id", containerId, ParameterType.UrlSegment);
            request.AddHeader("Authorization", "Bearer " + Credentials.TokenInternal);
            request.AddHeader("Content-Type", "application/vnd.api+json");
            return await client.ExecuteAsync(request);
        }


        public async Task<IRestResponse> PostIssuesAsync(string containerId, JObject data)
        {
            System.Net.Http.HttpClient httpClient = new System.Net.Http.HttpClient();
            Microsoft.AspNetCore.Http.Headers.RequestHeaders header =
           this.Request.GetTypedHeaders();
            Uri uriReferer = header.Referer;
            httpClient.BaseAddress = new Uri(uriReferer.AbsoluteUri);
            const string BASE_URL = "https://developer.api.autodesk.com";
            Credentials = await Credentials.FromSessionAsync(base.Request.Cookies,
           Response.Cookies);
            if (Credentials == null) { return null; }
            RestClient client = new RestClient(BASE_URL);
            RestRequest request = new RestRequest("/issues/v1/containers/{container_id}/quality-issues", RestSharp.Method.POST);
            request.AddParameter("container_id", containerId, ParameterType.UrlSegment);
            request.AddHeader("Authorization", "Bearer " + Credentials.TokenInternal);
            request.AddHeader("Content-Type", "application/vnd.api+json");
            request.AddParameter("text/json", Newtonsoft.Json.JsonConvert.SerializeObject(data),
           ParameterType.RequestBody);
            var resultado=await client.ExecuteAsync(request);
            return resultado;
        }

    }
}