﻿using Autodesk.Forge;
using Microsoft.AspNetCore.Mvc;
using Newtonsoft.Json.Linq;
using RestSharp;
using System.Threading.Tasks;

namespace TrabajoBIM360
{
    public class UserController : ControllerBase
    {
        [HttpGet]
        [Route("api/forge/user/profile")]
        public async Task<JObject> GetUserProfileAsync()
        {
            Credentials credentials = await Credentials.FromSessionAsync(Request.Cookies, Response.Cookies);
            if (credentials == null)
            {
                return null;
            }

            // the API SDK
            UserProfileApi userApi = new UserProfileApi();
            userApi.Configuration.AccessToken = credentials.TokenInternal;


            // get the user profile
            dynamic userProfile = await userApi.GetUserProfileAsync();

            // prepare a response with name & picture
            dynamic response = new JObject();
            response.name = string.Format("{0} {1}", userProfile.firstName, userProfile.lastName);
            response.picture = userProfile.profileImages.sizeX40;
            return response;
        }

        [HttpGet]
        [Route("api/forge/user/showAllUsers")]
        public async Task<JArray> showAllUsers(string hubId)
        {
            //Credentials credentials = await Credentials.FromSessionAsync(Request.Cookies, Response.Cookies);
            //if (credentials == null)
            //{
            //    return null;
            //}


            TwoLeggedApi oauth = new TwoLeggedApi();
            dynamic bearer = await oauth.AuthenticateAsync(Credentials.GetAppSetting("Forge_CLIENT_ID"),
            Credentials.GetAppSetting("Forge_CLIENT_SECRET"), "client_credentials", new Scope[] { Scope.AccountRead });


            RestClient client = new RestClient("https://developer.api.autodesk.com");
            //Region US
            RestRequest request = new RestRequest("/hq/v1/accounts/{account_id}/users", RestSharp.Method.GET);

            //Region EMEA
            // RestRequest request = new RestRequest("/hq/v1/regions/eu/accounts/{account_id}/users", RestSharp.Method.GET);

            //Podemos hacer este reemplazo, dado que al inicio de la aplicación nos quedamos solamente con los hub de BIM 360
            request.AddParameter("account_id", hubId.Replace("b.", string.Empty), ParameterType.UrlSegment);
            request.AddHeader("Authorization", "Bearer " + /*credentials.TokenPublic*/ bearer.access_token);
            IRestResponse usersResponse = await client.ExecuteTaskAsync(request);
            JArray users = JArray.Parse(usersResponse.Content);
            return users;
        }

    }
}
