﻿
function showIssues(issues) {
    var _this = this;
    var pushPinExtensionName = 'Autodesk.BIM360.Extension.PushPin';
    //remove the list of last time
    var pushPinExtension = _this.viewer.getExtension(pushPinExtensionName);
    pushPinExtension.removeAllItems();
    pushPinExtension.showAll();

    var pushpinDataArray = [];

    if (issues.length > 0) {
        _this.panel = new Autodesk.Viewing.UI.PropertyPanel(_this.viewer.container, 'bim360IssuePanel', 'BIM 360 Issues');
        _this.panel.setVisible('true');
    }

    issues.forEach(function (issue) {

        // if panel is NOT visible, exit the function
        if (!_this.panel.isVisible()) return;
        // show issue on panel
        if (_this.panel) {
            _this.panel.addProperty('Title', issue.attributes.title, 'Issue ' + issue.attributes.identifier);
            _this.panel.addProperty('Version', 'V' + issue.attributes.starting_version,
                'Issue ' + issue.attributes.identifier);
            _this.panel.addProperty('Created at', issue.attributes.created_at, 'Issue ' + issue.attributes.identifier);
            _this.panel.addProperty('Assigned to', issue.attributes.assigned_to_name, 'Issue ' + issue.attributes.identifier);
        }
        // add the pushpin
        var issueAttributes = issue.attributes;
        var pushpinAttributes = issue.attributes.pushpin_attributes;
        if (pushpinAttributes) {
            issue.type = issue.type.replace('quality_', '');

            pushpinDataArray.push({
                id: issue.id,
                label: issueAttributes.identifier,
                status: issue.type && issueAttributes.status.indexOf(issue.type) === -1 ?
                    `${issue.type}-${issueAttributes.status}` : issueAttributes.status,
                position: pushpinAttributes.location,
                type: issue.type,
                objectId: pushpinAttributes.object_id,
                viewerState: pushpinAttributes.viewer_state
            });
        }
    })

    pushPinExtension.loadItems(pushpinDataArray);

}


function createIssue(target_urn, starting_version, containerId) {
    var _this = this;
    var pushPinExtensionName = 'Autodesk.BIM360.Extension.PushPin';
    var pushPinExtension = _this.viewer.getExtension(pushPinExtensionName);
    var issueLabel = prompt("Introduzca el nombre de la issue: ");
    if (issueLabel === null) return;
    // start asking for the push location
    pushPinExtension.startCreateItem({ label: issueLabel, status: 'open', type: 'issues' });
    // prepare to end creation...
    pushPinExtension.pushPinManager.addEventListener('pushpin.created', function (e) {
        pushPinExtension.pushPinManager.removeEventListener('pushpin.created', arguments.callee);
        pushPinExtension.endCreateItem();
        //we seem to have to get it from the first item of pushpin list, which is always the latest new one
        var issue =
            pushPinExtension.getItemById(pushPinExtension.pushPinManager.pushPinList[0].itemData.id);
        if (issue === null) return; // safeguard
        var data = {
            type: 'quality_issues',//issue.type,
            attributes: {
                title: issue.label,
                // You now need to extract the ``status`` (``open``) from the pushpin object.
                status: issue.status.split('-')[1] || issue.status,
                target_urn: target_urn,
                starting_version: starting_version,
                sheet_metadata: { // `viewerApp.selectedItem` references the current sheet
                    is3D: this.viewer.model.is3D,
                    sheetGuid: this.viewer.model.getDocumentNode().data.guid,
                    sheetName: this.viewer.model.getDocumentNode().data.name
                },
                pushpin_attributes: { // Data about the pushpin
                    type: 'TwoDVectorPushpin', // This is the only type currently available
                    object_id: issue.objectId, // (Only for 3D models) The object the pushpin is situated on.
                    location: issue.position, // The x, y, z coordinates of the pushpin.
                    viewer_state: issue.viewerState // The current viewer state. For example, angle, camera, zoom.
                },
            }
        };
        // submit data
        var urn = btoa(target_urn.split('?')[0]);
        jQuery.post({
            url: '/api/forge/issues/' + containerId + '/issues/' + urn,
            contentType: 'application/json',
            data: JSON.stringify({ data: data }),
            success: function (res) {
                console.log(res);
                showQualityIssues();
            },
            error: function (err) {
                console.log(err.responseText);
                pushPinExtension.pushPinManager.removeItemById('0');
                alert('Cannot create issue');
            }
        });
    });
}
