﻿var viewer;

// @urn the model to show
// @viewablesId which viewables to show, applies to BIM 360 Plans folder
function launchViewer(urn, viewableId) {
    var options = {
        env: 'AutodeskProduction',
        language:'enu',
        getAccessToken: getForgeToken,
        api: 'derivativeV2' + (atob(urn.replace('_', '/')).indexOf('emea') > -1 ? '_EU' : '') // handle BIM 360 US and EU regions
    };

    Autodesk.Viewing.Initializer(options, () => {
        viewer = new Autodesk.Viewing.GuiViewer3D(document.getElementById('forgeViewer'), {
            extensions: ['Autodesk.DocumentBrowser'] });
        viewer.start();
        var documentId = 'urn:' + urn;
        Autodesk.Viewing.Document.load(documentId, onDocumentLoadSuccess, onDocumentLoadFailure);
    });

    function onDocumentLoadSuccess(doc) {
        // if a viewableId was specified, load that view, otherwise the default view
        var viewables = (viewableId ? doc.getRoot().findByGuid(viewableId) : doc.getRoot().getDefaultGeometry());
        doc.downloadAecModelData();
        viewer.loadDocumentNode(doc, viewables).then(i => {
            viewer.loadExtension('Autodesk.Viewing.ZoomWindow');
            if (viewer.model.is3d()) {
                viewer.loadExtension('Autodesk.AEC.Minimap3DExtension');
                viewer.loadExtension("Autodesk.AEC.LevelsExtension");
            }
            if (viewer.model.is2d()) {
                viewer.loadExtension('Autodesk.BIM360.Minimap');
            }
            viewer.loadExtension('Botones');
            viewer.loadExtension('MenuContextual');
            var extensionOptions = {
                hideIssuesButton: true,
                hideRfisButton: true,
                hideFieldIssuesButton: true,
            };
            viewer.loadExtension('Autodesk.BIM360.Extension.PushPin', extensionOptions);
            // any additional action here?
        });
    }

    function onDocumentLoadFailure(viewerErrorCode, viewerErrorMsg) {
        console.error('onDocumentLoadFailure() - errorCode:' + viewerErrorCode + '\n- errorMessage:' + viewerErrorMsg);
    }
}

function getForgeToken(callback) {
    fetch('/api/forge/oauth/token').then(res => {
        res.json().then(data => {
            callback(data.access_token, data.expires_in);
        });
    });
}
