/* 
 * Forge SDK
 *
 * The Forge Platform contains an expanding collection of web service components that can be used with Autodesk cloud-based products or your own technologies. Take advantage of Autodesk’s expertise in design and engineering.
 *
 * OpenAPI spec version: 0.1.0
 * Contact: forge.help@autodesk.com
 * Generated by: https://github.com/swagger-api/swagger-codegen.git
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.Linq;
using RestSharp;
using Newtonsoft.Json.Linq;
using Autodesk.Forge.Client;
using Autodesk.Forge.Model;

namespace Autodesk.Forge
{
    /// <summary>
    /// Represents a collection of functions to interact with the API endpoints
    /// </summary>
    public interface IFoldersApi : IApiAccessor
    {
        #region Synchronous Operations
        /// <summary>
        /// Returns a collection of items recursively in subfolders of any project accessible to you. 
        /// </summary>
        /// <param name="projectId">Poject Id</param>
        /// <param name="folderId">Folder Id</param>
        /// <param name="pageNumber">Page number</param>
        ///<param name="filterName">filter data what contains the string</param>
        /// <param name="filterFileType">filter data by file extension</param>
        /// <returns>Json Api Collection</returns>
        ApiResponse< /*JsonApiCollection*/dynamic> SearchFolderContents(string projectId, string folderId, int? pageNumber = null, List<string> filterName = null, List<string> filterFileType = null);

        /// <summary>
        /// 
        /// </summary>
        /// <remarks>
        /// Returns the folder by ID for any folder within a given project. All folders or sub-folders within a project are associated with their own unique ID, including the root folder. 
        /// </remarks>
        /// <exception cref="Autodesk.Forge.Client.ApiException">Thrown when fails to make API call</exception>
        /// <param name="projectId">the &#x60;project id&#x60;</param>
        /// <param name="folderId">the &#x60;folder id&#x60;</param>
        /// <returns>Folder</returns>
        /*Folder*/ dynamic GetFolder (string projectId, string folderId);

        /// <summary>
        /// 
        /// </summary>
        /// <remarks>
        /// Returns the folder by ID for any folder within a given project. All folders or sub-folders within a project are associated with their own unique ID, including the root folder. 
        /// </remarks>
        /// <exception cref="Autodesk.Forge.Client.ApiException">Thrown when fails to make API call</exception>
        /// <param name="projectId">the &#x60;project id&#x60;</param>
        /// <param name="folderId">the &#x60;folder id&#x60;</param>
        /// <returns>ApiResponse of Folder</returns>
        ApiResponse</*Folder*/dynamic> GetFolderWithHttpInfo (string projectId, string folderId);
        /// <summary>
        /// 
        /// </summary>
        /// <remarks>
        /// Returns a collection of items and folders within a folder. Items represent word documents, fusion design files, drawings, spreadsheets, etc. 
        /// </remarks>
        /// <exception cref="Autodesk.Forge.Client.ApiException">Thrown when fails to make API call</exception>
        /// <param name="projectId">the &#x60;project id&#x60;</param>
        /// <param name="folderId">the &#x60;folder id&#x60;</param>
        /// <param name="filterType">filter by the &#x60;type&#x60; of the &#x60;ref&#x60; target (optional)</param>
        /// <param name="filterId">filter by the &#x60;id&#x60; of the &#x60;ref&#x60; target (optional)</param>
        /// <param name="filterExtensionType">filter by the extension type (optional)</param>
        /// <param name="pageNumber">specify the page number (optional)</param>
        /// <param name="pageLimit">specify the maximal number of elements per page (optional)</param>
        /// <returns>JsonApiCollection</returns>
        /*JsonApiCollection*/dynamic GetFolderContents (string projectId, string folderId, List<string> filterType = null, List<string> filterId = null, List<string> filterExtensionType = null, int? pageNumber = null, int? pageLimit = null);

        /// <summary>
        /// 
        /// </summary>
        /// <remarks>
        /// Returns a collection of items and folders within a folder. Items represent word documents, fusion design files, drawings, spreadsheets, etc. 
        /// </remarks>
        /// <exception cref="Autodesk.Forge.Client.ApiException">Thrown when fails to make API call</exception>
        /// <param name="projectId">the &#x60;project id&#x60;</param>
        /// <param name="folderId">the &#x60;folder id&#x60;</param>
        /// <param name="filterType">filter by the &#x60;type&#x60; of the &#x60;ref&#x60; target (optional)</param>
        /// <param name="filterId">filter by the &#x60;id&#x60; of the &#x60;ref&#x60; target (optional)</param>
        /// <param name="filterExtensionType">filter by the extension type (optional)</param>
        /// <param name="pageNumber">specify the page number (optional)</param>
        /// <param name="pageLimit">specify the maximal number of elements per page (optional)</param>
        /// <returns>ApiResponse of JsonApiCollection</returns>
        ApiResponse</*JsonApiCollection*/dynamic> GetFolderContentsWithHttpInfo (string projectId, string folderId, List<string> filterType = null, List<string> filterId = null, List<string> filterExtensionType = null, int? pageNumber = null, int? pageLimit = null);
        /// <summary>
        /// 
        /// </summary>
        /// <remarks>
        /// Returns the parent folder (if it exists). In a project, subfolders and resource items are stored under a folder except the root folder which does not have a parent of its own. 
        /// </remarks>
        /// <exception cref="Autodesk.Forge.Client.ApiException">Thrown when fails to make API call</exception>
        /// <param name="projectId">the &#x60;project id&#x60;</param>
        /// <param name="folderId">the &#x60;folder id&#x60;</param>
        /// <returns>Folder</returns>
        /*Folder*/dynamic GetFolderParent (string projectId, string folderId);

        /// <summary>
        /// 
        /// </summary>
        /// <remarks>
        /// Returns the parent folder (if it exists). In a project, subfolders and resource items are stored under a folder except the root folder which does not have a parent of its own. 
        /// </remarks>
        /// <exception cref="Autodesk.Forge.Client.ApiException">Thrown when fails to make API call</exception>
        /// <param name="projectId">the &#x60;project id&#x60;</param>
        /// <param name="folderId">the &#x60;folder id&#x60;</param>
        /// <returns>ApiResponse of Folder</returns>
        ApiResponse</*Folder*/dynamic> GetFolderParentWithHttpInfo (string projectId, string folderId);
        /// <summary>
        /// 
        /// </summary>
        /// <remarks>
        /// Returns the resources (&#x60;items&#x60;, &#x60;folders&#x60;, and &#x60;versions&#x60;) which have a custom relationship with the given &#x60;folder_id&#x60;. Custom relationships can be established between a folder and other resources within the &#39;data&#39; domain service (folders, items, and versions). 
        /// </remarks>
        /// <exception cref="Autodesk.Forge.Client.ApiException">Thrown when fails to make API call</exception>
        /// <param name="projectId">the &#x60;project id&#x60;</param>
        /// <param name="folderId">the &#x60;folder id&#x60;</param>
        /// <param name="filterType">filter by the &#x60;type&#x60; of the &#x60;ref&#x60; target (optional)</param>
        /// <param name="filterId">filter by the &#x60;id&#x60; of the &#x60;ref&#x60; target (optional)</param>
        /// <param name="filterExtensionType">filter by the extension type (optional)</param>
        /// <returns>JsonApiCollection</returns>
        /*JsonApiCollection*/dynamic GetFolderRefs (string projectId, string folderId, List<string> filterType = null, List<string> filterId = null, List<string> filterExtensionType = null);

        /// <summary>
        /// 
        /// </summary>
        /// <remarks>
        /// Returns the resources (&#x60;items&#x60;, &#x60;folders&#x60;, and &#x60;versions&#x60;) which have a custom relationship with the given &#x60;folder_id&#x60;. Custom relationships can be established between a folder and other resources within the &#39;data&#39; domain service (folders, items, and versions). 
        /// </remarks>
        /// <exception cref="Autodesk.Forge.Client.ApiException">Thrown when fails to make API call</exception>
        /// <param name="projectId">the &#x60;project id&#x60;</param>
        /// <param name="folderId">the &#x60;folder id&#x60;</param>
        /// <param name="filterType">filter by the &#x60;type&#x60; of the &#x60;ref&#x60; target (optional)</param>
        /// <param name="filterId">filter by the &#x60;id&#x60; of the &#x60;ref&#x60; target (optional)</param>
        /// <param name="filterExtensionType">filter by the extension type (optional)</param>
        /// <returns>ApiResponse of JsonApiCollection</returns>
        ApiResponse</*JsonApiCollection*/dynamic> GetFolderRefsWithHttpInfo (string projectId, string folderId, List<string> filterType = null, List<string> filterId = null, List<string> filterExtensionType = null);
        /// <summary>
        /// 
        /// </summary>
        /// <remarks>
        /// Returns the custom relationships that are associated to the given &#x60;folder_id&#x60;. Custom relationships can be established between a folder and other resources within the &#39;data&#39; domain service (folders, items, and versions). 
        /// </remarks>
        /// <exception cref="Autodesk.Forge.Client.ApiException">Thrown when fails to make API call</exception>
        /// <param name="projectId">the &#x60;project id&#x60;</param>
        /// <param name="folderId">the &#x60;folder id&#x60;</param>
        /// <param name="filterType">filter by the &#x60;type&#x60; of the &#x60;ref&#x60; target (optional)</param>
        /// <param name="filterId">filter by the &#x60;id&#x60; of the &#x60;ref&#x60; target (optional)</param>
        /// <param name="filterRefType">filter by &#x60;refType&#x60; (optional)</param>
        /// <param name="filterDirection">filter by the direction of the reference (optional)</param>
        /// <param name="filterExtensionType">filter by the extension type (optional)</param>
        /// <returns>Refs</returns>
        /*Refs*/dynamic GetFolderRelationshipsRefs (string projectId, string folderId, List<string> filterType = null, List<string> filterId = null, List<string> filterRefType = null, string filterDirection = null, List<string> filterExtensionType = null);

        /// <summary>
        /// 
        /// </summary>
        /// <remarks>
        /// Returns the custom relationships that are associated to the given &#x60;folder_id&#x60;. Custom relationships can be established between a folder and other resources within the &#39;data&#39; domain service (folders, items, and versions). 
        /// </remarks>
        /// <exception cref="Autodesk.Forge.Client.ApiException">Thrown when fails to make API call</exception>
        /// <param name="projectId">the &#x60;project id&#x60;</param>
        /// <param name="folderId">the &#x60;folder id&#x60;</param>
        /// <param name="filterType">filter by the &#x60;type&#x60; of the &#x60;ref&#x60; target (optional)</param>
        /// <param name="filterId">filter by the &#x60;id&#x60; of the &#x60;ref&#x60; target (optional)</param>
        /// <param name="filterRefType">filter by &#x60;refType&#x60; (optional)</param>
        /// <param name="filterDirection">filter by the direction of the reference (optional)</param>
        /// <param name="filterExtensionType">filter by the extension type (optional)</param>
        /// <returns>ApiResponse of Refs</returns>
        ApiResponse</*Refs*/dynamic> GetFolderRelationshipsRefsWithHttpInfo (string projectId, string folderId, List<string> filterType = null, List<string> filterId = null, List<string> filterRefType = null, string filterDirection = null, List<string> filterExtensionType = null);

        /// <summary>
        ///  Deletes or renames a folder in the &#x60;data&#x60; domain service
        /// </summary>
        /// <exception cref="Autodesk.Forge.Client.ApiException">Thrown when fails to make API call</exception>
        /// <param name="projectId">the &#x60;project id&#x60;</param>
        /// <param name="body">describe the folder to be updated</param>
        /// <returns>ApiResponse of PatchFolder</returns>
        dynamic PatchFolder(string projectId, PatchFolder body);

        /// <summary>
        /// 
        /// </summary>
        /// <remarks>
        ///  Deletes or renames a folder in the &#x60;data&#x60; domain service
        /// </remarks>
        /// <exception cref="Autodesk.Forge.Client.ApiException">Thrown when fails to make API call</exception>
        /// <param name="projectId">the &#x60;project id&#x60;</param>
        /// <param name="body">describe the folder to be updated</param>
        /// <returns>ApiResponse of Object(void)</returns>
        ApiResponse<dynamic> PatchFolderWithHttpInfo(string projectId, PatchFolder body);

        /// <summary>
        /// 
        /// </summary>
        /// <remarks>
        /// Creates a new folder in the &#x60;data&#x60; domain service
        /// </remarks>
        /// <exception cref="Autodesk.Forge.Client.ApiException">Thrown when fails to make API call</exception>
        /// <param name="projectId">the &#x60;project id&#x60;</param>
        /// <param name="body">describe the folder to be created</param>
        /// <returns></returns>
        dynamic PostFolder (string projectId, CreateFolder body);

        /// <summary>
        /// 
        /// </summary>
        /// <remarks>
        /// Creates a new folder in the &#x60;data&#x60; domain service
        /// </remarks>
        /// <exception cref="Autodesk.Forge.Client.ApiException">Thrown when fails to make API call</exception>
        /// <param name="projectId">the &#x60;project id&#x60;</param>
        /// <param name="body">describe the folder to be created</param>
        /// <returns>ApiResponse of Object(void)</returns>
        ApiResponse<dynamic> PostFolderWithHttpInfo (string projectId, CreateFolder body);
        /// <summary>
        /// 
        /// </summary>
        /// <remarks>
        /// Creates a custom relationship between a folder and another resource within the &#39;data&#39; domain service (folder, item, or version). 
        /// </remarks>
        /// <exception cref="Autodesk.Forge.Client.ApiException">Thrown when fails to make API call</exception>
        /// <param name="projectId">the &#x60;project id&#x60;</param>
        /// <param name="folderId">the &#x60;folder id&#x60;</param>
        /// <param name="body">describe the ref to be created</param>
        /// <returns></returns>
        dynamic PostFolderRelationshipsRef (string projectId, string folderId, CreateRef body);

        /// <summary>
        /// 
        /// </summary>
        /// <remarks>
        /// Creates a custom relationship between a folder and another resource within the &#39;data&#39; domain service (folder, item, or version). 
        /// </remarks>
        /// <exception cref="Autodesk.Forge.Client.ApiException">Thrown when fails to make API call</exception>
        /// <param name="projectId">the &#x60;project id&#x60;</param>
        /// <param name="folderId">the &#x60;folder id&#x60;</param>
        /// <param name="body">describe the ref to be created</param>
        /// <returns>ApiResponse of Object(void)</returns>
        ApiResponse<dynamic> PostFolderRelationshipsRefWithHttpInfo (string projectId, string folderId, CreateRef body);
        #endregion Synchronous Operations
        #region Asynchronous Operations    
        /// <summary>
        /// Returns a collection of items recursively in subfolders of any project accessible to you. 
        /// </summary>
        /// <param name="projectId">Poject Id</param>
        /// <param name="folderId">Folder Id</param>
        /// <param name="pageNumber">Page number</param>
        ///<param name="filterName">filter data what contains the string</param>
        /// <param name="filterFileType">filter data by file extension</param>
        /// <returns>Json Api Collection</returns>
        System.Threading.Tasks.Task<ApiResponse</*JsonApiCollection*/dynamic>> SearchFolderContentsAsync(string projectId, string folderId, int? pageNumber = null,List<string> filterName = null, List<string> filterFileType = null);

        /// <summary>
        /// 
        /// </summary>
        /// <remarks>
        /// Returns the folder by ID for any folder within a given project. All folders or sub-folders within a project are associated with their own unique ID, including the root folder. 
        /// </remarks>
        /// <exception cref="Autodesk.Forge.Client.ApiException">Thrown when fails to make API call</exception>
        /// <param name="projectId">the &#x60;project id&#x60;</param>
        /// <param name="folderId">the &#x60;folder id&#x60;</param>
        /// <returns>Task of Folder</returns>
        System.Threading.Tasks.Task</*Folder*/dynamic> GetFolderAsync (string projectId, string folderId);

        /// <summary>
        /// 
        /// </summary>
        /// <remarks>
        /// Returns the folder by ID for any folder within a given project. All folders or sub-folders within a project are associated with their own unique ID, including the root folder. 
        /// </remarks>
        /// <exception cref="Autodesk.Forge.Client.ApiException">Thrown when fails to make API call</exception>
        /// <param name="projectId">the &#x60;project id&#x60;</param>
        /// <param name="folderId">the &#x60;folder id&#x60;</param>
        /// <returns>Task of ApiResponse (Folder)</returns>
        System.Threading.Tasks.Task<ApiResponse</*Folder*/dynamic>> GetFolderAsyncWithHttpInfo (string projectId, string folderId);
        /// <summary>
        /// 
        /// </summary>
        /// <remarks>
        /// Returns a collection of items and folders within a folder. Items represent word documents, fusion design files, drawings, spreadsheets, etc. 
        /// </remarks>
        /// <exception cref="Autodesk.Forge.Client.ApiException">Thrown when fails to make API call</exception>
        /// <param name="projectId">the &#x60;project id&#x60;</param>
        /// <param name="folderId">the &#x60;folder id&#x60;</param>
        /// <param name="filterType">filter by the &#x60;type&#x60; of the &#x60;ref&#x60; target (optional)</param>
        /// <param name="filterId">filter by the &#x60;id&#x60; of the &#x60;ref&#x60; target (optional)</param>
        /// <param name="filterExtensionType">filter by the extension type (optional)</param>
        /// <param name="pageNumber">specify the page number (optional)</param>
        /// <param name="pageLimit">specify the maximal number of elements per page (optional)</param>
        /// <returns>Task of JsonApiCollection</returns>
        System.Threading.Tasks.Task</*JsonApiCollection*/dynamic> GetFolderContentsAsync (string projectId, string folderId, List<string> filterType = null, List<string> filterId = null, List<string> filterExtensionType = null, int? pageNumber = null, int? pageLimit = null, List<string> includeHidden = null);

        /// <summary>
        /// 
        /// </summary>
        /// <remarks>
        /// Returns a collection of items and folders within a folder. Items represent word documents, fusion design files, drawings, spreadsheets, etc. 
        /// </remarks>
        /// <exception cref="Autodesk.Forge.Client.ApiException">Thrown when fails to make API call</exception>
        /// <param name="projectId">the &#x60;project id&#x60;</param>
        /// <param name="folderId">the &#x60;folder id&#x60;</param>
        /// <param name="filterType">filter by the &#x60;type&#x60; of the &#x60;ref&#x60; target (optional)</param>
        /// <param name="filterId">filter by the &#x60;id&#x60; of the &#x60;ref&#x60; target (optional)</param>
        /// <param name="filterExtensionType">filter by the extension type (optional)</param>
        /// <param name="pageNumber">specify the page number (optional)</param>
        /// <param name="pageLimit">specify the maximal number of elements per page (optional)</param>
        /// <returns>Task of ApiResponse (JsonApiCollection)</returns>
        System.Threading.Tasks.Task<ApiResponse</*JsonApiCollection*/dynamic>> GetFolderContentsAsyncWithHttpInfo (string projectId, string folderId, List<string> filterType = null, List<string> filterId = null, List<string> filterExtensionType = null, int? pageNumber = null, int? pageLimit = null, List<string> includeHidden = null);
        /// <summary>
        /// 
        /// </summary>
        /// <remarks>
        /// Returns the parent folder (if it exists). In a project, subfolders and resource items are stored under a folder except the root folder which does not have a parent of its own. 
        /// </remarks>
        /// <exception cref="Autodesk.Forge.Client.ApiException">Thrown when fails to make API call</exception>
        /// <param name="projectId">the &#x60;project id&#x60;</param>
        /// <param name="folderId">the &#x60;folder id&#x60;</param>
        /// <returns>Task of Folder</returns>
        System.Threading.Tasks.Task</*Folder*/dynamic> GetFolderParentAsync (string projectId, string folderId);

        /// <summary>
        /// 
        /// </summary>
        /// <remarks>
        /// Returns the parent folder (if it exists). In a project, subfolders and resource items are stored under a folder except the root folder which does not have a parent of its own. 
        /// </remarks>
        /// <exception cref="Autodesk.Forge.Client.ApiException">Thrown when fails to make API call</exception>
        /// <param name="projectId">the &#x60;project id&#x60;</param>
        /// <param name="folderId">the &#x60;folder id&#x60;</param>
        /// <returns>Task of ApiResponse (Folder)</returns>
        System.Threading.Tasks.Task<ApiResponse</*Folder*/dynamic>> GetFolderParentAsyncWithHttpInfo (string projectId, string folderId);
        /// <summary>
        /// 
        /// </summary>
        /// <remarks>
        /// Returns the resources (&#x60;items&#x60;, &#x60;folders&#x60;, and &#x60;versions&#x60;) which have a custom relationship with the given &#x60;folder_id&#x60;. Custom relationships can be established between a folder and other resources within the &#39;data&#39; domain service (folders, items, and versions). 
        /// </remarks>
        /// <exception cref="Autodesk.Forge.Client.ApiException">Thrown when fails to make API call</exception>
        /// <param name="projectId">the &#x60;project id&#x60;</param>
        /// <param name="folderId">the &#x60;folder id&#x60;</param>
        /// <param name="filterType">filter by the &#x60;type&#x60; of the &#x60;ref&#x60; target (optional)</param>
        /// <param name="filterId">filter by the &#x60;id&#x60; of the &#x60;ref&#x60; target (optional)</param>
        /// <param name="filterExtensionType">filter by the extension type (optional)</param>
        /// <returns>Task of JsonApiCollection</returns>
        System.Threading.Tasks.Task</*JsonApiCollection*/dynamic> GetFolderRefsAsync (string projectId, string folderId, List<string> filterType = null, List<string> filterId = null, List<string> filterExtensionType = null);

        /// <summary>
        /// 
        /// </summary>
        /// <remarks>
        /// Returns the resources (&#x60;items&#x60;, &#x60;folders&#x60;, and &#x60;versions&#x60;) which have a custom relationship with the given &#x60;folder_id&#x60;. Custom relationships can be established between a folder and other resources within the &#39;data&#39; domain service (folders, items, and versions). 
        /// </remarks>
        /// <exception cref="Autodesk.Forge.Client.ApiException">Thrown when fails to make API call</exception>
        /// <param name="projectId">the &#x60;project id&#x60;</param>
        /// <param name="folderId">the &#x60;folder id&#x60;</param>
        /// <param name="filterType">filter by the &#x60;type&#x60; of the &#x60;ref&#x60; target (optional)</param>
        /// <param name="filterId">filter by the &#x60;id&#x60; of the &#x60;ref&#x60; target (optional)</param>
        /// <param name="filterExtensionType">filter by the extension type (optional)</param>
        /// <returns>Task of ApiResponse (JsonApiCollection)</returns>
        System.Threading.Tasks.Task<ApiResponse</*JsonApiCollection*/dynamic>> GetFolderRefsAsyncWithHttpInfo (string projectId, string folderId, List<string> filterType = null, List<string> filterId = null, List<string> filterExtensionType = null);
        /// <summary>
        /// 
        /// </summary>
        /// <remarks>
        /// Returns the custom relationships that are associated to the given &#x60;folder_id&#x60;. Custom relationships can be established between a folder and other resources within the &#39;data&#39; domain service (folders, items, and versions). 
        /// </remarks>
        /// <exception cref="Autodesk.Forge.Client.ApiException">Thrown when fails to make API call</exception>
        /// <param name="projectId">the &#x60;project id&#x60;</param>
        /// <param name="folderId">the &#x60;folder id&#x60;</param>
        /// <param name="filterType">filter by the &#x60;type&#x60; of the &#x60;ref&#x60; target (optional)</param>
        /// <param name="filterId">filter by the &#x60;id&#x60; of the &#x60;ref&#x60; target (optional)</param>
        /// <param name="filterRefType">filter by &#x60;refType&#x60; (optional)</param>
        /// <param name="filterDirection">filter by the direction of the reference (optional)</param>
        /// <param name="filterExtensionType">filter by the extension type (optional)</param>
        /// <returns>Task of Refs</returns>
        System.Threading.Tasks.Task</*Refs*/dynamic> GetFolderRelationshipsRefsAsync (string projectId, string folderId, List<string> filterType = null, List<string> filterId = null, List<string> filterRefType = null, string filterDirection = null, List<string> filterExtensionType = null);

        /// <summary>
        /// 
        /// </summary>
        /// <remarks>
        /// Returns the custom relationships that are associated to the given &#x60;folder_id&#x60;. Custom relationships can be established between a folder and other resources within the &#39;data&#39; domain service (folders, items, and versions). 
        /// </remarks>
        /// <exception cref="Autodesk.Forge.Client.ApiException">Thrown when fails to make API call</exception>
        /// <param name="projectId">the &#x60;project id&#x60;</param>
        /// <param name="folderId">the &#x60;folder id&#x60;</param>
        /// <param name="filterType">filter by the &#x60;type&#x60; of the &#x60;ref&#x60; target (optional)</param>
        /// <param name="filterId">filter by the &#x60;id&#x60; of the &#x60;ref&#x60; target (optional)</param>
        /// <param name="filterRefType">filter by &#x60;refType&#x60; (optional)</param>
        /// <param name="filterDirection">filter by the direction of the reference (optional)</param>
        /// <param name="filterExtensionType">filter by the extension type (optional)</param>
        /// <returns>Task of ApiResponse (Refs)</returns>
        System.Threading.Tasks.Task<ApiResponse</*Refs*/dynamic>> GetFolderRelationshipsRefsAsyncWithHttpInfo (string projectId, string folderId, List<string> filterType = null, List<string> filterId = null, List<string> filterRefType = null, string filterDirection = null, List<string> filterExtensionType = null);

        /// <summary>
        /// </summary>
        /// <remarks>
        ///  Deletes or renames a folder in the &#x60;data&#x60; domain service
        /// </remarks>
        /// <exception cref="Autodesk.Forge.Client.ApiException">Thrown when fails to make API call</exception>
        /// <param name="projectId">the &#x60;project id&#x60;</param>
        /// <param name="body">describe the folder to be updated</param>
        /// <returns></returns>
        System.Threading.Tasks.Task<dynamic> PatchFolderAsync(string projectId, PatchFolder body);


        /// <summary>
        /// </summary>
        /// <remarks>
        ///  Deletes or renames a folder in the &#x60;data&#x60; domain service
        /// </remarks>
        /// <exception cref="Autodesk.Forge.Client.ApiException">Thrown when fails to make API call</exception>
        /// <param name="projectId">the &#x60;project id&#x60;</param>
        /// <param name="body">describe the folder to be updated</param>
        /// <returns></returns>
        System.Threading.Tasks.Task<ApiResponse<dynamic>> PatchFolderAsyncWithHttpInfo(string projectId, PatchFolder body);


        /// <summary>
        /// 
        /// </summary>
        /// <remarks>
        /// Creates a new folder in the &#x60;data&#x60; domain service
        /// </remarks>
        /// <exception cref="Autodesk.Forge.Client.ApiException">Thrown when fails to make API call</exception>
        /// <param name="projectId">the &#x60;project id&#x60;</param>
        /// <param name="body">describe the folder to be created</param>
        /// <returns>Task of void</returns>
        System.Threading.Tasks.Task<dynamic> PostFolderAsync (string projectId, CreateFolder body);

        /// <summary>
        /// 
        /// </summary>
        /// <remarks>
        /// Creates a new folder in the &#x60;data&#x60; domain service
        /// </remarks>
        /// <exception cref="Autodesk.Forge.Client.ApiException">Thrown when fails to make API call</exception>
        /// <param name="projectId">the &#x60;project id&#x60;</param>
        /// <param name="body">describe the folder to be created</param>
        /// <returns>Task of ApiResponse</returns>
        System.Threading.Tasks.Task<ApiResponse<dynamic>> PostFolderAsyncWithHttpInfo (string projectId, CreateFolder body);
        /// <summary>
        /// 
        /// </summary>
        /// <remarks>
        /// Creates a custom relationship between a folder and another resource within the &#39;data&#39; domain service (folder, item, or version). 
        /// </remarks>
        /// <exception cref="Autodesk.Forge.Client.ApiException">Thrown when fails to make API call</exception>
        /// <param name="projectId">the &#x60;project id&#x60;</param>
        /// <param name="folderId">the &#x60;folder id&#x60;</param>
        /// <param name="body">describe the ref to be created</param>
        /// <returns>Task of void</returns>
        System.Threading.Tasks.Task<dynamic> PostFolderRelationshipsRefAsync (string projectId, string folderId, CreateRef body);

        /// <summary>
        /// 
        /// </summary>
        /// <remarks>
        /// Creates a custom relationship between a folder and another resource within the &#39;data&#39; domain service (folder, item, or version). 
        /// </remarks>
        /// <exception cref="Autodesk.Forge.Client.ApiException">Thrown when fails to make API call</exception>
        /// <param name="projectId">the &#x60;project id&#x60;</param>
        /// <param name="folderId">the &#x60;folder id&#x60;</param>
        /// <param name="body">describe the ref to be created</param>
        /// <returns>Task of ApiResponse</returns>
        System.Threading.Tasks.Task<ApiResponse<dynamic>> PostFolderRelationshipsRefAsyncWithHttpInfo (string projectId, string folderId, CreateRef body);
        #endregion Asynchronous Operations
    }

    /// <summary>
    /// Represents a collection of functions to interact with the API endpoints
    /// </summary>
    public partial class FoldersApi : IFoldersApi
    {
        private Autodesk.Forge.Client.ExceptionFactory _exceptionFactory = (name, response) => null;

        /// <summary>
        /// Initializes a new instance of the <see cref="FoldersApi"/> class.
        /// </summary>
        /// <returns></returns>
        public FoldersApi(String basePath)
        {
            this.Configuration = new Configuration(new ApiClient(basePath));

            ExceptionFactory = Autodesk.Forge.Client.Configuration.DefaultExceptionFactory;

            // ensure API client has configuration ready
            if (Configuration.ApiClient.Configuration == null)
            {
                this.Configuration.ApiClient.Configuration = this.Configuration;
            }
        }

        /// <summary>
        /// Initializes a new instance of the <see cref="FoldersApi"/> class
        /// using Configuration object
        /// </summary>
        /// <param name="configuration">An instance of Configuration</param>
        /// <returns></returns>
        public FoldersApi(Configuration configuration = null)
        {
            if (configuration == null) // use the default one in Configuration
                this.Configuration = Configuration.Default;
            else
                this.Configuration = configuration;

            ExceptionFactory = Autodesk.Forge.Client.Configuration.DefaultExceptionFactory;

            // ensure API client has configuration ready
            if (Configuration.ApiClient.Configuration == null)
            {
                this.Configuration.ApiClient.Configuration = this.Configuration;
            }
        }

        /// <summary>
        /// Gets the base path of the API client.
        /// </summary>
        /// <value>The base path</value>
        public String GetBasePath()
        {
            return this.Configuration.ApiClient.RestClient.BaseUrl.ToString();
        }

        /// <summary>
        /// Sets the base path of the API client.
        /// </summary>
        /// <value>The base path</value>
        [Obsolete("SetBasePath is deprecated, please do 'Configuration.ApiClient = new ApiClient(\"http://new-path\")' instead.")]
        public void SetBasePath(String basePath)
        {
            // do nothing
        }

        /// <summary>
        /// Gets or sets the configuration object
        /// </summary>
        /// <value>An instance of the Configuration</value>
        public Configuration Configuration {get; set;}

        /// <summary>
        /// Provides a factory method hook for the creation of exceptions.
        /// </summary>
        public Autodesk.Forge.Client.ExceptionFactory ExceptionFactory
        {
            get
            {
                if (_exceptionFactory != null && _exceptionFactory.GetInvocationList().Length > 1)
                {
                    throw new InvalidOperationException("Multicast delegate for ExceptionFactory is unsupported.");
                }
                return _exceptionFactory;
            }
            set { _exceptionFactory = value; }
        }

        /// <summary>
        /// Gets the default header.
        /// </summary>
        /// <returns>Dictionary of HTTP header</returns>
        [Obsolete("DefaultHeader is deprecated, please use Configuration.DefaultHeader instead.")]
        public Dictionary<String, String> DefaultHeader()
        {
            return this.Configuration.DefaultHeader;
        }

        /// <summary>
        /// Add default header.
        /// </summary>
        /// <param name="key">Header field name.</param>
        /// <param name="value">Header field value.</param>
        /// <returns></returns>
        [Obsolete("AddDefaultHeader is deprecated, please use Configuration.AddDefaultHeader instead.")]
        public void AddDefaultHeader(string key, string value)
        {
            this.Configuration.AddDefaultHeader(key, value);
        }

        /// <summary>
        ///  Returns the folder by ID for any folder within a given project. All folders or sub-folders within a project are associated with their own unique ID, including the root folder. 
        /// </summary>
        /// <exception cref="Autodesk.Forge.Client.ApiException">Thrown when fails to make API call</exception>
        /// <param name="projectId">the &#x60;project id&#x60;</param>
        /// <param name="folderId">the &#x60;folder id&#x60;</param>
        /// <returns>Folder</returns>
        public /*Folder*/dynamic GetFolder (string projectId, string folderId)
        {
             ApiResponse</*Folder*/dynamic> localVarResponse = GetFolderWithHttpInfo(projectId, folderId);
             return localVarResponse.Data;
        }

        /// <summary>
        ///  Returns the folder by ID for any folder within a given project. All folders or sub-folders within a project are associated with their own unique ID, including the root folder. 
        /// </summary>
        /// <exception cref="Autodesk.Forge.Client.ApiException">Thrown when fails to make API call</exception>
        /// <param name="projectId">the &#x60;project id&#x60;</param>
        /// <param name="folderId">the &#x60;folder id&#x60;</param>
        /// <returns>ApiResponse of Folder</returns>
        public ApiResponse< /*Folder*/dynamic> GetFolderWithHttpInfo(string projectId, string folderId)
        {
            // verify the required parameter 'projectId' is set
            if (projectId == null)
                throw new ApiException(400, "Missing required parameter 'projectId' when calling FoldersApi->GetFolder");
            // verify the required parameter 'folderId' is set
            if (folderId == null)
                throw new ApiException(400, "Missing required parameter 'folderId' when calling FoldersApi->GetFolder");

            var localVarPath = "/data/v1/projects/{project_id}/folders/{folder_id}";
            var localVarPathParams = new Dictionary<String, String>();
            var localVarQueryParams = new Dictionary<String, String>();
            var localVarHeaderParams = new Dictionary<String, String>(Configuration.DefaultHeader);
            var localVarFormParams = new Dictionary<String, String>();
            var localVarFileParams = new Dictionary<String, FileParameter>();
            Object localVarPostBody = null;

            // to determine the Content-Type header
            String[] localVarHttpContentTypes = new String[] {
                "application/vnd.api+json"
            };
            String localVarHttpContentType = Configuration.ApiClient.SelectHeaderContentType(localVarHttpContentTypes);

            // to determine the Accept header
            String[] localVarHttpHeaderAccepts = new String[] {
                "application/vnd.api+json",
                "application/json"
            };
            String localVarHttpHeaderAccept = Configuration.ApiClient.SelectHeaderAccept(localVarHttpHeaderAccepts);
            if (localVarHttpHeaderAccept != null)
                localVarHeaderParams.Add("Accept", localVarHttpHeaderAccept);

            // set "format" to json by default
            // e.g. /pet/{petId}.{format} becomes /pet/{petId}.json
            localVarPathParams.Add("format", "json");
            if (projectId != null) localVarPathParams.Add("project_id", Configuration.ApiClient.ParameterToString(projectId)); // path parameter
            if (folderId != null) localVarPathParams.Add("folder_id", Configuration.ApiClient.ParameterToString(folderId)); // path parameter

            // authentication (oauth2_access_code) required
            // oauth required
            if (!String.IsNullOrEmpty(Configuration.AccessToken))
            {
                localVarHeaderParams["Authorization"] = "Bearer " + Configuration.AccessToken;
            }

            // make the HTTP request
            IRestResponse localVarResponse = (IRestResponse)Configuration.ApiClient.CallApi(localVarPath,
                Method.GET, localVarQueryParams, localVarPostBody, localVarHeaderParams, localVarFormParams, localVarFileParams,
                localVarPathParams, localVarHttpContentType);

            int localVarStatusCode = (int)localVarResponse.StatusCode;

            if (ExceptionFactory != null)
            {
                Exception exception = ExceptionFactory("GetFolder", localVarResponse);
                if (exception != null) throw exception;
            }

            return new ApiResponse</*Folder*/dynamic>(localVarStatusCode,
                localVarResponse.Headers.ToDictionary(x => x.Name, x => x.Value.ToString()),
                /*(Folder)*/ Configuration.ApiClient.Deserialize(localVarResponse, typeof(Folder)));

        }

        /// <summary>
        /// Returns a collection of items recursively in subfolders of any project accessible to you. 
        /// </summary>
        /// <param name="projectId">Poject Id</param>
        /// <param name="folderId">Folder Id</param>
        /// <param name="pageNumber">Page number</param>
        ///<param name="filterName">filter data what contains the string</param>
        /// <param name="filterFileType">filter data by file extension</param>
        /// <returns>Json Api Collection</returns>
        public ApiResponse< /*JsonApiCollection*/dynamic> SearchFolderContents(string projectId, string folderId, int? pageNumber = null, List<string> filterName = null, List<string> filterFileType = null)
        {
            // verify the required parameter 'projectId' is set
            if (projectId == null)
                throw new ApiException(400, "Missing required parameter 'projectId' when calling FoldersApi->SearchFolderContents");
            // verify the required parameter 'folderId' is set
            if (folderId == null)
                throw new ApiException(400, "Missing required parameter 'folderId' when calling FoldersApi->SearchFolderContents");

            var localVarPath = "/data/v1/projects/{project_id}/folders/{folder_id}/search";
            var localVarPathParams = new Dictionary<String, String>();
            var localVarQueryParams = new Dictionary<String, String>();
            var localVarHeaderParams = new Dictionary<String, String>(Configuration.DefaultHeader);
            var localVarFormParams = new Dictionary<String, String>();
            var localVarFileParams = new Dictionary<String, FileParameter>();
            Object localVarPostBody = null;

            // to determine the Content-Type header
            String[] localVarHttpContentTypes = new String[] {
                "application/vnd.api+json"
            };
            String localVarHttpContentType = Configuration.ApiClient.SelectHeaderContentType(localVarHttpContentTypes);

            // to determine the Accept header
            String[] localVarHttpHeaderAccepts = new String[] {
                "application/vnd.api+json",
                "application/json"
            };
            String localVarHttpHeaderAccept = Configuration.ApiClient.SelectHeaderAccept(localVarHttpHeaderAccepts);
            if (localVarHttpHeaderAccept != null)
                localVarHeaderParams.Add("Accept", localVarHttpHeaderAccept);

            // set "format" to json by default
            // e.g. /pet/{petId}.{format} becomes /pet/{petId}.json
            localVarPathParams.Add("format", "json");
            localVarPathParams.Add("project_id", Configuration.ApiClient.ParameterToString(projectId)); // path parameter
            localVarPathParams.Add("folder_id", Configuration.ApiClient.ParameterToString(folderId)); // path parameter
            if (pageNumber != null) localVarQueryParams.Add("page[number]", Configuration.ApiClient.ParameterToString(pageNumber)); // query parameter
            if (filterFileType != null) localVarQueryParams.Add("filter[fileType]", Configuration.ApiClient.ParameterToString(filterFileType)); // query parameter
            if (filterName != null) localVarQueryParams.Add("filter[name]-contains", Configuration.ApiClient.ParameterToString(filterName)); // query parameter
            
            // authentication (oauth2_access_code) required
            // oauth required
            if (!String.IsNullOrEmpty(Configuration.AccessToken))
            {
                localVarHeaderParams["Authorization"] = "Bearer " + Configuration.AccessToken;
            }

            // make the HTTP request
            IRestResponse localVarResponse = (IRestResponse)Configuration.ApiClient.CallApi(localVarPath,
                Method.GET, localVarQueryParams, localVarPostBody, localVarHeaderParams, localVarFormParams, localVarFileParams,
                localVarPathParams, localVarHttpContentType);

            int localVarStatusCode = (int)localVarResponse.StatusCode;

            if (ExceptionFactory != null)
            {
                Exception exception = ExceptionFactory("SearchFolderContents", localVarResponse);
                if (exception != null) throw exception;
            }

            return new ApiResponse</*JsonApiCollection*/dynamic>(localVarStatusCode,
                localVarResponse.Headers.ToDictionary(x => x.Name, x => x.Value.ToString()),
                /*(JsonApiCollection)*/ Configuration.ApiClient.Deserialize(localVarResponse, typeof(JsonApiCollection)));

        }

        /// <summary>
        /// Returns a collection of items recursively in subfolders of any project accessible to you. 
        /// </summary>
        /// <param name="projectId">Poject Id</param>
        /// <param name="folderId">Folder Id</param>
        /// <param name="pageNumber">Page number, starts with 0</param>
        ///<param name="filterName">filter data what contains the string</param>
        /// <param name="filterFileType">filter data by file extension</param>
        /// <returns>Json Api Collection</returns>
        public async System.Threading.Tasks.Task<ApiResponse</*JsonApiCollection*/dynamic>> SearchFolderContentsAsync(string projectId, string folderId, int? pageNumber = 0,List<string> filterName = null, List<string> filterFileType = null)
        {
            // verify the required parameter 'projectId' is set
            if (projectId == null)
                throw new ApiException(400, "Missing required parameter 'projectId' when calling FoldersApi->SearchFolderContentsAsync");
            // verify the required parameter 'folderId' is set
            if (folderId == null)
                throw new ApiException(400, "Missing required parameter 'folderId' when calling FoldersApi->SearchFolderContentsAsync");

            const string localVarPath = "/data/v1/projects/{project_id}/folders/{folder_id}/search";
            var localVarPathParams = new Dictionary<String, String>();
            var localVarQueryParams = new Dictionary<String, String>();
            var localVarHeaderParams = new Dictionary<String, String>(Configuration.DefaultHeader);
            var localVarFormParams = new Dictionary<String, String>();
            var localVarFileParams = new Dictionary<String, FileParameter>();

            // to determine the Content-Type header
            var localVarHttpContentTypes = new String[] {
                "application/vnd.api+json"
            };
            var localVarHttpContentType = Configuration.ApiClient.SelectHeaderContentType(localVarHttpContentTypes);

            // to determine the Accept header
            var localVarHttpHeaderAccepts = new String[] {
                "application/vnd.api+json",
                "application/json"
            };
            var localVarHttpHeaderAccept = Configuration.ApiClient.SelectHeaderAccept(localVarHttpHeaderAccepts);
            if (localVarHttpHeaderAccept != null)
                localVarHeaderParams.Add("Accept", localVarHttpHeaderAccept);

            localVarPathParams.Add("format", "json");
            localVarPathParams.Add("project_id", Configuration.ApiClient.ParameterToString(projectId)); // path parameter
            localVarPathParams.Add("folder_id", Configuration.ApiClient.ParameterToString(folderId)); // path parameter
            if (pageNumber != null) localVarQueryParams.Add("page[number]", Configuration.ApiClient.ParameterToString(pageNumber)); // query parameter
            if (filterFileType != null) localVarQueryParams.Add("filter[fileType]", Configuration.ApiClient.ParameterToString(filterFileType)); // query parameter
            if (filterName != null) localVarQueryParams.Add("filter[name]-contains", Configuration.ApiClient.ParameterToString(filterName)); // query parameter
           
            // authentication (oauth2_access_code) required
            // oauth required
            if (!String.IsNullOrEmpty(Configuration.AccessToken))
            {
                localVarHeaderParams["Authorization"] = "Bearer " + Configuration.AccessToken;
            }

            // make the HTTP request
            var localVarResponse = (IRestResponse)await Configuration.ApiClient.CallApiAsync(localVarPath,
                Method.GET, localVarQueryParams, null, localVarHeaderParams, localVarFormParams, localVarFileParams,
                localVarPathParams, localVarHttpContentType);

            var localVarStatusCode = (int)localVarResponse.StatusCode;

            if (ExceptionFactory != null)
            {
                Exception exception = ExceptionFactory("SearchFolderContentsAsync", localVarResponse);
                if (exception != null) throw exception;
            }

            return new ApiResponse</*JsonApiCollection*/dynamic>(localVarStatusCode,
                localVarResponse.Headers.ToDictionary(x => x.Name, x => x.Value.ToString()),
                /*(JsonApiCollection)*/ Configuration.ApiClient.Deserialize(localVarResponse, typeof(JsonApiCollection)));

        }

        /// <summary>
        ///  Returns the folder by ID for any folder within a given project. All folders or sub-folders within a project are associated with their own unique ID, including the root folder. 
        /// </summary>
        /// <exception cref="Autodesk.Forge.Client.ApiException">Thrown when fails to make API call</exception>
        /// <param name="projectId">the &#x60;project id&#x60;</param>
        /// <param name="folderId">the &#x60;folder id&#x60;</param>
        /// <returns>Task of Folder</returns>
        public async System.Threading.Tasks.Task</*Folder*/dynamic> GetFolderAsync (string projectId, string folderId)
        {
             ApiResponse</*Folder*/dynamic> localVarResponse = await GetFolderAsyncWithHttpInfo(projectId, folderId);
             return localVarResponse.Data;

        }

        /// <summary>
        ///  Returns the folder by ID for any folder within a given project. All folders or sub-folders within a project are associated with their own unique ID, including the root folder. 
        /// </summary>
        /// <exception cref="Autodesk.Forge.Client.ApiException">Thrown when fails to make API call</exception>
        /// <param name="projectId">the &#x60;project id&#x60;</param>
        /// <param name="folderId">the &#x60;folder id&#x60;</param>
        /// <returns>Task of ApiResponse (Folder)</returns>
        public async System.Threading.Tasks.Task<ApiResponse</*Folder*/dynamic>> GetFolderAsyncWithHttpInfo (string projectId, string folderId)
        {
            // verify the required parameter 'projectId' is set
            if (projectId == null)
                throw new ApiException(400, "Missing required parameter 'projectId' when calling FoldersApi->GetFolder");
            // verify the required parameter 'folderId' is set
            if (folderId == null)
                throw new ApiException(400, "Missing required parameter 'folderId' when calling FoldersApi->GetFolder");

            var localVarPath = "/data/v1/projects/{project_id}/folders/{folder_id}";
            var localVarPathParams = new Dictionary<String, String>();
            var localVarQueryParams = new Dictionary<String, String>();
            var localVarHeaderParams = new Dictionary<String, String>(Configuration.DefaultHeader);
            var localVarFormParams = new Dictionary<String, String>();
            var localVarFileParams = new Dictionary<String, FileParameter>();
            Object localVarPostBody = null;

            // to determine the Content-Type header
            String[] localVarHttpContentTypes = new String[] {
                "application/vnd.api+json"
            };
            String localVarHttpContentType = Configuration.ApiClient.SelectHeaderContentType(localVarHttpContentTypes);

            // to determine the Accept header
            String[] localVarHttpHeaderAccepts = new String[] {
                "application/vnd.api+json", 
                "application/json"
            };
            String localVarHttpHeaderAccept = Configuration.ApiClient.SelectHeaderAccept(localVarHttpHeaderAccepts);
            if (localVarHttpHeaderAccept != null)
                localVarHeaderParams.Add("Accept", localVarHttpHeaderAccept);

            // set "format" to json by default
            // e.g. /pet/{petId}.{format} becomes /pet/{petId}.json
            localVarPathParams.Add("format", "json");
            if (projectId != null) localVarPathParams.Add("project_id", Configuration.ApiClient.ParameterToString(projectId)); // path parameter
            if (folderId != null) localVarPathParams.Add("folder_id", Configuration.ApiClient.ParameterToString(folderId)); // path parameter

            // authentication (oauth2_access_code) required
            // oauth required
            if (!String.IsNullOrEmpty(Configuration.AccessToken))
            {
                localVarHeaderParams["Authorization"] = "Bearer " + Configuration.AccessToken;
            }

            // make the HTTP request
            IRestResponse localVarResponse = (IRestResponse) await Configuration.ApiClient.CallApiAsync(localVarPath,
                Method.GET, localVarQueryParams, localVarPostBody, localVarHeaderParams, localVarFormParams, localVarFileParams,
                localVarPathParams, localVarHttpContentType);

            int localVarStatusCode = (int) localVarResponse.StatusCode;

            if (ExceptionFactory != null)
            {
                Exception exception = ExceptionFactory("GetFolder", localVarResponse);
                if (exception != null) throw exception;
            }

            return new ApiResponse</*Folder*/dynamic>(localVarStatusCode,
                localVarResponse.Headers.ToDictionary(x => x.Name, x => x.Value.ToString()),
                /*(Folder)*/ Configuration.ApiClient.Deserialize(localVarResponse, typeof(Folder)));
            
        }

        /// <summary>
        ///  Returns a collection of items and folders within a folder. Items represent word documents, fusion design files, drawings, spreadsheets, etc. 
        /// </summary>
        /// <exception cref="Autodesk.Forge.Client.ApiException">Thrown when fails to make API call</exception>
        /// <param name="projectId">the &#x60;project id&#x60;</param>
        /// <param name="folderId">the &#x60;folder id&#x60;</param>
        /// <param name="filterType">filter by the &#x60;type&#x60; of the &#x60;ref&#x60; target (optional)</param>
        /// <param name="filterId">filter by the &#x60;id&#x60; of the &#x60;ref&#x60; target (optional)</param>
        /// <param name="filterExtensionType">filter by the extension type (optional)</param>
        /// <param name="pageNumber">specify the page number (optional)</param>
        /// <param name="pageLimit">specify the maximal number of elements per page (optional)</param>
        /// <returns>JsonApiCollection</returns>
        public /*JsonApiCollection*/dynamic GetFolderContents (string projectId, string folderId, List<string> filterType = null, List<string> filterId = null, List<string> filterExtensionType = null, int? pageNumber = null, int? pageLimit = null)
        {
             ApiResponse</*JsonApiCollection*/dynamic> localVarResponse = GetFolderContentsWithHttpInfo(projectId, folderId, filterType, filterId, filterExtensionType, pageNumber, pageLimit);
             return localVarResponse.Data;
        }

        /// <summary>
        ///  Returns a collection of items and folders within a folder. Items represent word documents, fusion design files, drawings, spreadsheets, etc. 
        /// </summary>
        /// <exception cref="Autodesk.Forge.Client.ApiException">Thrown when fails to make API call</exception>
        /// <param name="projectId">the &#x60;project id&#x60;</param>
        /// <param name="folderId">the &#x60;folder id&#x60;</param>
        /// <param name="filterType">filter by the &#x60;type&#x60; of the &#x60;ref&#x60; target (optional)</param>
        /// <param name="filterId">filter by the &#x60;id&#x60; of the &#x60;ref&#x60; target (optional)</param>
        /// <param name="filterExtensionType">filter by the extension type (optional)</param>
        /// <param name="pageNumber">specify the page number (optional)</param>
        /// <param name="pageLimit">specify the maximal number of elements per page (optional)</param>
        /// <returns>ApiResponse of JsonApiCollection</returns>
        public ApiResponse< /*JsonApiCollection*/dynamic > GetFolderContentsWithHttpInfo (string projectId, string folderId, List<string> filterType = null, List<string> filterId = null, List<string> filterExtensionType = null, int? pageNumber = null, int? pageLimit = null)
        {
            // verify the required parameter 'projectId' is set
            if (projectId == null)
                throw new ApiException(400, "Missing required parameter 'projectId' when calling FoldersApi->GetFolderContents");
            // verify the required parameter 'folderId' is set
            if (folderId == null)
                throw new ApiException(400, "Missing required parameter 'folderId' when calling FoldersApi->GetFolderContents");

            var localVarPath = "/data/v1/projects/{project_id}/folders/{folder_id}/contents";
            var localVarPathParams = new Dictionary<String, String>();
            var localVarQueryParams = new Dictionary<String, String>();
            var localVarHeaderParams = new Dictionary<String, String>(Configuration.DefaultHeader);
            var localVarFormParams = new Dictionary<String, String>();
            var localVarFileParams = new Dictionary<String, FileParameter>();
            Object localVarPostBody = null;

            // to determine the Content-Type header
            String[] localVarHttpContentTypes = new String[] {
                "application/vnd.api+json"
            };
            String localVarHttpContentType = Configuration.ApiClient.SelectHeaderContentType(localVarHttpContentTypes);

            // to determine the Accept header
            String[] localVarHttpHeaderAccepts = new String[] {
                "application/vnd.api+json", 
                "application/json"
            };
            String localVarHttpHeaderAccept = Configuration.ApiClient.SelectHeaderAccept(localVarHttpHeaderAccepts);
            if (localVarHttpHeaderAccept != null)
                localVarHeaderParams.Add("Accept", localVarHttpHeaderAccept);

            // set "format" to json by default
            // e.g. /pet/{petId}.{format} becomes /pet/{petId}.json
            localVarPathParams.Add("format", "json");
            if (projectId != null) localVarPathParams.Add("project_id", Configuration.ApiClient.ParameterToString(projectId)); // path parameter
            if (folderId != null) localVarPathParams.Add("folder_id", Configuration.ApiClient.ParameterToString(folderId)); // path parameter
            if (filterType != null) localVarQueryParams.Add("filter[type]", Configuration.ApiClient.ParameterToString(filterType)); // query parameter
            if (filterId != null) localVarQueryParams.Add("filter[id]", Configuration.ApiClient.ParameterToString(filterId)); // query parameter
            if (filterExtensionType != null) localVarQueryParams.Add("filter[extension.type]", Configuration.ApiClient.ParameterToString(filterExtensionType)); // query parameter
            if (pageNumber != null) localVarQueryParams.Add("page[number]", Configuration.ApiClient.ParameterToString(pageNumber)); // query parameter
            if (pageLimit != null) localVarQueryParams.Add("page[limit]", Configuration.ApiClient.ParameterToString(pageLimit)); // query parameter

            // authentication (oauth2_access_code) required
            // oauth required
            if (!String.IsNullOrEmpty(Configuration.AccessToken))
            {
                localVarHeaderParams["Authorization"] = "Bearer " + Configuration.AccessToken;
            }

            // make the HTTP request
            IRestResponse localVarResponse = (IRestResponse) Configuration.ApiClient.CallApi(localVarPath,
                Method.GET, localVarQueryParams, localVarPostBody, localVarHeaderParams, localVarFormParams, localVarFileParams,
                localVarPathParams, localVarHttpContentType);

            int localVarStatusCode = (int) localVarResponse.StatusCode;

            if (ExceptionFactory != null)
            {
                Exception exception = ExceptionFactory("GetFolderContents", localVarResponse);
                if (exception != null) throw exception;
            }

            return new ApiResponse</*JsonApiCollection*/dynamic>(localVarStatusCode,
                localVarResponse.Headers.ToDictionary(x => x.Name, x => x.Value.ToString()),
                /*(JsonApiCollection)*/ Configuration.ApiClient.Deserialize(localVarResponse, typeof(JsonApiCollection)));
            
        }

        /// <summary>
        ///  Returns a collection of items and folders within a folder. Items represent word documents, fusion design files, drawings, spreadsheets, etc. 
        /// </summary>
        /// <exception cref="Autodesk.Forge.Client.ApiException">Thrown when fails to make API call</exception>
        /// <param name="projectId">the &#x60;project id&#x60;</param>
        /// <param name="folderId">the &#x60;folder id&#x60;</param>
        /// <param name="filterType">filter by the &#x60;type&#x60; of the &#x60;ref&#x60; target (optional)</param>
        /// <param name="filterId">filter by the &#x60;id&#x60; of the &#x60;ref&#x60; target (optional)</param>
        /// <param name="filterExtensionType">filter by the extension type (optional)</param>
        /// <param name="pageNumber">specify the page number (optional)</param>
        /// <param name="pageLimit">specify the maximal number of elements per page (optional)</param>
        /// <returns>Task of JsonApiCollection</returns>
        public async System.Threading.Tasks.Task</*JsonApiCollection*/dynamic> GetFolderContentsAsync (string projectId, string folderId, List<string> filterType = null, List<string> filterId = null, List<string> filterExtensionType = null, int? pageNumber = null, int? pageLimit = null, List<string> includeHidden = null)
        {
             ApiResponse</*JsonApiCollection*/dynamic> localVarResponse = await GetFolderContentsAsyncWithHttpInfo(projectId, folderId, filterType, filterId, filterExtensionType, pageNumber, pageLimit,includeHidden);
             return localVarResponse.Data;
        }

        /// <summary>
        ///  Returns a collection of items and folders within a folder. Items represent word documents, fusion design files, drawings, spreadsheets, etc. 
        /// </summary>
        /// <exception cref="Autodesk.Forge.Client.ApiException">Thrown when fails to make API call</exception>
        /// <param name="projectId">the &#x60;project id&#x60;</param>
        /// <param name="folderId">the &#x60;folder id&#x60;</param>
        /// <param name="filterType">filter by the &#x60;type&#x60; of the &#x60;ref&#x60; target (optional)</param>
        /// <param name="filterId">filter by the &#x60;id&#x60; of the &#x60;ref&#x60; target (optional)</param>
        /// <param name="filterExtensionType">filter by the extension type (optional)</param>
        /// <param name="pageNumber">specify the page number (optional)</param>
        /// <param name="pageLimit">specify the maximal number of elements per page (optional)</param>
        /// <returns>Task of ApiResponse (JsonApiCollection)</returns>
        public async System.Threading.Tasks.Task<ApiResponse</*JsonApiCollection*/dynamic>> GetFolderContentsAsyncWithHttpInfo (string projectId, string folderId, List<string> filterType = null, List<string> filterId = null, List<string> filterExtensionType = null, int? pageNumber = null, int? pageLimit = null, List<string> includeHidden = null)
        {
            // verify the required parameter 'projectId' is set
            if (projectId == null)
                throw new ApiException(400, "Missing required parameter 'projectId' when calling FoldersApi->GetFolderContents");
            // verify the required parameter 'folderId' is set
            if (folderId == null)
                throw new ApiException(400, "Missing required parameter 'folderId' when calling FoldersApi->GetFolderContents");

            var localVarPath = "/data/v1/projects/{project_id}/folders/{folder_id}/contents";
            var localVarPathParams = new Dictionary<String, String>();
            var localVarQueryParams = new Dictionary<String, String>();
            var localVarHeaderParams = new Dictionary<String, String>(Configuration.DefaultHeader);
            var localVarFormParams = new Dictionary<String, String>();
            var localVarFileParams = new Dictionary<String, FileParameter>();
            Object localVarPostBody = null;

            // to determine the Content-Type header
            String[] localVarHttpContentTypes = new String[] {
                "application/vnd.api+json"
            };
            String localVarHttpContentType = Configuration.ApiClient.SelectHeaderContentType(localVarHttpContentTypes);

            // to determine the Accept header
            String[] localVarHttpHeaderAccepts = new String[] {
                "application/vnd.api+json", 
                "application/json"
            };
            String localVarHttpHeaderAccept = Configuration.ApiClient.SelectHeaderAccept(localVarHttpHeaderAccepts);
            if (localVarHttpHeaderAccept != null)
                localVarHeaderParams.Add("Accept", localVarHttpHeaderAccept);

            // set "format" to json by default
            // e.g. /pet/{petId}.{format} becomes /pet/{petId}.json
            localVarPathParams.Add("format", "json");
            if (projectId != null) localVarPathParams.Add("project_id", Configuration.ApiClient.ParameterToString(projectId)); // path parameter
            if (folderId != null) localVarPathParams.Add("folder_id", Configuration.ApiClient.ParameterToString(folderId)); // path parameter
            if (filterType != null) localVarQueryParams.Add("filter[type]", Configuration.ApiClient.ParameterToString(filterType)); // query parameter
            if (filterId != null) localVarQueryParams.Add("filter[id]", Configuration.ApiClient.ParameterToString(filterId)); // query parameter
            if (filterExtensionType != null) localVarQueryParams.Add("filter[extension.type]", Configuration.ApiClient.ParameterToString(filterExtensionType)); // query parameter
            if (pageNumber != null) localVarQueryParams.Add("page[number]", Configuration.ApiClient.ParameterToString(pageNumber)); // query parameter
            if (pageLimit != null) localVarQueryParams.Add("page[limit]", Configuration.ApiClient.ParameterToString(pageLimit)); // query parameter
            if (includeHidden != null) localVarQueryParams.Add("filter[hidden]", Configuration.ApiClient.ParameterToString(includeHidden)); // query parameter

            // authentication (oauth2_access_code) required
            // oauth required
            if (!String.IsNullOrEmpty(Configuration.AccessToken))
            {
                localVarHeaderParams["Authorization"] = "Bearer " + Configuration.AccessToken;
            }

            // make the HTTP request
            IRestResponse localVarResponse = (IRestResponse) await Configuration.ApiClient.CallApiAsync(localVarPath,
                Method.GET, localVarQueryParams, localVarPostBody, localVarHeaderParams, localVarFormParams, localVarFileParams,
                localVarPathParams, localVarHttpContentType);

            int localVarStatusCode = (int) localVarResponse.StatusCode;

            if (ExceptionFactory != null)
            {
                Exception exception = ExceptionFactory("GetFolderContents", localVarResponse);
                if (exception != null) throw exception;
            }

            return new ApiResponse</*JsonApiCollection*/dynamic>(localVarStatusCode,
                localVarResponse.Headers.ToDictionary(x => x.Name, x => x.Value.ToString()),
                /*(JsonApiCollection)*/ Configuration.ApiClient.Deserialize(localVarResponse, typeof(JsonApiCollection)));
            
        }

        /// <summary>
        ///  Returns the parent folder (if it exists). In a project, subfolders and resource items are stored under a folder except the root folder which does not have a parent of its own. 
        /// </summary>
        /// <exception cref="Autodesk.Forge.Client.ApiException">Thrown when fails to make API call</exception>
        /// <param name="projectId">the &#x60;project id&#x60;</param>
        /// <param name="folderId">the &#x60;folder id&#x60;</param>
        /// <returns>Folder</returns>
        public /*Folder*/dynamic GetFolderParent (string projectId, string folderId)
        {
             ApiResponse</*Folder*/dynamic> localVarResponse = GetFolderParentWithHttpInfo(projectId, folderId);
             return localVarResponse.Data;
        }

        /// <summary>
        ///  Returns the parent folder (if it exists). In a project, subfolders and resource items are stored under a folder except the root folder which does not have a parent of its own. 
        /// </summary>
        /// <exception cref="Autodesk.Forge.Client.ApiException">Thrown when fails to make API call</exception>
        /// <param name="projectId">the &#x60;project id&#x60;</param>
        /// <param name="folderId">the &#x60;folder id&#x60;</param>
        /// <returns>ApiResponse of Folder</returns>
        public ApiResponse< /*Folder*/dynamic > GetFolderParentWithHttpInfo (string projectId, string folderId)
        {
            // verify the required parameter 'projectId' is set
            if (projectId == null)
                throw new ApiException(400, "Missing required parameter 'projectId' when calling FoldersApi->GetFolderParent");
            // verify the required parameter 'folderId' is set
            if (folderId == null)
                throw new ApiException(400, "Missing required parameter 'folderId' when calling FoldersApi->GetFolderParent");

            var localVarPath = "/data/v1/projects/{project_id}/folders/{folder_id}/parent";
            var localVarPathParams = new Dictionary<String, String>();
            var localVarQueryParams = new Dictionary<String, String>();
            var localVarHeaderParams = new Dictionary<String, String>(Configuration.DefaultHeader);
            var localVarFormParams = new Dictionary<String, String>();
            var localVarFileParams = new Dictionary<String, FileParameter>();
            Object localVarPostBody = null;

            // to determine the Content-Type header
            String[] localVarHttpContentTypes = new String[] {
                "application/vnd.api+json"
            };
            String localVarHttpContentType = Configuration.ApiClient.SelectHeaderContentType(localVarHttpContentTypes);

            // to determine the Accept header
            String[] localVarHttpHeaderAccepts = new String[] {
                "application/vnd.api+json", 
                "application/json"
            };
            String localVarHttpHeaderAccept = Configuration.ApiClient.SelectHeaderAccept(localVarHttpHeaderAccepts);
            if (localVarHttpHeaderAccept != null)
                localVarHeaderParams.Add("Accept", localVarHttpHeaderAccept);

            // set "format" to json by default
            // e.g. /pet/{petId}.{format} becomes /pet/{petId}.json
            localVarPathParams.Add("format", "json");
            if (projectId != null) localVarPathParams.Add("project_id", Configuration.ApiClient.ParameterToString(projectId)); // path parameter
            if (folderId != null) localVarPathParams.Add("folder_id", Configuration.ApiClient.ParameterToString(folderId)); // path parameter

            // authentication (oauth2_access_code) required
            // oauth required
            if (!String.IsNullOrEmpty(Configuration.AccessToken))
            {
                localVarHeaderParams["Authorization"] = "Bearer " + Configuration.AccessToken;
            }

            // make the HTTP request
            IRestResponse localVarResponse = (IRestResponse) Configuration.ApiClient.CallApi(localVarPath,
                Method.GET, localVarQueryParams, localVarPostBody, localVarHeaderParams, localVarFormParams, localVarFileParams,
                localVarPathParams, localVarHttpContentType);

            int localVarStatusCode = (int) localVarResponse.StatusCode;

            if (ExceptionFactory != null)
            {
                Exception exception = ExceptionFactory("GetFolderParent", localVarResponse);
                if (exception != null) throw exception;
            }

            return new ApiResponse</*Folder*/dynamic>(localVarStatusCode,
                localVarResponse.Headers.ToDictionary(x => x.Name, x => x.Value.ToString()),
                /*(Folder)*/ Configuration.ApiClient.Deserialize(localVarResponse, typeof(Folder)));
            
        }

        /// <summary>
        ///  Returns the parent folder (if it exists). In a project, subfolders and resource items are stored under a folder except the root folder which does not have a parent of its own. 
        /// </summary>
        /// <exception cref="Autodesk.Forge.Client.ApiException">Thrown when fails to make API call</exception>
        /// <param name="projectId">the &#x60;project id&#x60;</param>
        /// <param name="folderId">the &#x60;folder id&#x60;</param>
        /// <returns>Task of Folder</returns>
        public async System.Threading.Tasks.Task</*Folder*/dynamic> GetFolderParentAsync (string projectId, string folderId)
        {
             ApiResponse</*Folder*/dynamic> localVarResponse = await GetFolderParentAsyncWithHttpInfo(projectId, folderId);
             return localVarResponse.Data;

        }

        /// <summary>
        ///  Returns the parent folder (if it exists). In a project, subfolders and resource items are stored under a folder except the root folder which does not have a parent of its own. 
        /// </summary>
        /// <exception cref="Autodesk.Forge.Client.ApiException">Thrown when fails to make API call</exception>
        /// <param name="projectId">the &#x60;project id&#x60;</param>
        /// <param name="folderId">the &#x60;folder id&#x60;</param>
        /// <returns>Task of ApiResponse (Folder)</returns>
        public async System.Threading.Tasks.Task<ApiResponse</*Folder*/dynamic>> GetFolderParentAsyncWithHttpInfo (string projectId, string folderId)
        {
            // verify the required parameter 'projectId' is set
            if (projectId == null)
                throw new ApiException(400, "Missing required parameter 'projectId' when calling FoldersApi->GetFolderParent");
            // verify the required parameter 'folderId' is set
            if (folderId == null)
                throw new ApiException(400, "Missing required parameter 'folderId' when calling FoldersApi->GetFolderParent");

            var localVarPath = "/data/v1/projects/{project_id}/folders/{folder_id}/parent";
            var localVarPathParams = new Dictionary<String, String>();
            var localVarQueryParams = new Dictionary<String, String>();
            var localVarHeaderParams = new Dictionary<String, String>(Configuration.DefaultHeader);
            var localVarFormParams = new Dictionary<String, String>();
            var localVarFileParams = new Dictionary<String, FileParameter>();
            Object localVarPostBody = null;

            // to determine the Content-Type header
            String[] localVarHttpContentTypes = new String[] {
                "application/vnd.api+json"
            };
            String localVarHttpContentType = Configuration.ApiClient.SelectHeaderContentType(localVarHttpContentTypes);

            // to determine the Accept header
            String[] localVarHttpHeaderAccepts = new String[] {
                "application/vnd.api+json", 
                "application/json"
            };
            String localVarHttpHeaderAccept = Configuration.ApiClient.SelectHeaderAccept(localVarHttpHeaderAccepts);
            if (localVarHttpHeaderAccept != null)
                localVarHeaderParams.Add("Accept", localVarHttpHeaderAccept);

            // set "format" to json by default
            // e.g. /pet/{petId}.{format} becomes /pet/{petId}.json
            localVarPathParams.Add("format", "json");
            if (projectId != null) localVarPathParams.Add("project_id", Configuration.ApiClient.ParameterToString(projectId)); // path parameter
            if (folderId != null) localVarPathParams.Add("folder_id", Configuration.ApiClient.ParameterToString(folderId)); // path parameter

            // authentication (oauth2_access_code) required
            // oauth required
            if (!String.IsNullOrEmpty(Configuration.AccessToken))
            {
                localVarHeaderParams["Authorization"] = "Bearer " + Configuration.AccessToken;
            }

            // make the HTTP request
            IRestResponse localVarResponse = (IRestResponse) await Configuration.ApiClient.CallApiAsync(localVarPath,
                Method.GET, localVarQueryParams, localVarPostBody, localVarHeaderParams, localVarFormParams, localVarFileParams,
                localVarPathParams, localVarHttpContentType);

            int localVarStatusCode = (int) localVarResponse.StatusCode;

            if (ExceptionFactory != null)
            {
                Exception exception = ExceptionFactory("GetFolderParent", localVarResponse);
                if (exception != null) throw exception;
            }

            return new ApiResponse</*Folder*/dynamic>(localVarStatusCode,
                localVarResponse.Headers.ToDictionary(x => x.Name, x => x.Value.ToString()),
                /*(Folder)*/ Configuration.ApiClient.Deserialize(localVarResponse, typeof(Folder)));
            
        }

        /// <summary>
        ///  Returns the resources (&#x60;items&#x60;, &#x60;folders&#x60;, and &#x60;versions&#x60;) which have a custom relationship with the given &#x60;folder_id&#x60;. Custom relationships can be established between a folder and other resources within the &#39;data&#39; domain service (folders, items, and versions). 
        /// </summary>
        /// <exception cref="Autodesk.Forge.Client.ApiException">Thrown when fails to make API call</exception>
        /// <param name="projectId">the &#x60;project id&#x60;</param>
        /// <param name="folderId">the &#x60;folder id&#x60;</param>
        /// <param name="filterType">filter by the &#x60;type&#x60; of the &#x60;ref&#x60; target (optional)</param>
        /// <param name="filterId">filter by the &#x60;id&#x60; of the &#x60;ref&#x60; target (optional)</param>
        /// <param name="filterExtensionType">filter by the extension type (optional)</param>
        /// <returns>JsonApiCollection</returns>
        public /*JsonApiCollection*/dynamic GetFolderRefs (string projectId, string folderId, List<string> filterType = null, List<string> filterId = null, List<string> filterExtensionType = null)
        {
             ApiResponse</*JsonApiCollection*/dynamic> localVarResponse = GetFolderRefsWithHttpInfo(projectId, folderId, filterType, filterId, filterExtensionType);
             return localVarResponse.Data;
        }

        /// <summary>
        ///  Returns the resources (&#x60;items&#x60;, &#x60;folders&#x60;, and &#x60;versions&#x60;) which have a custom relationship with the given &#x60;folder_id&#x60;. Custom relationships can be established between a folder and other resources within the &#39;data&#39; domain service (folders, items, and versions). 
        /// </summary>
        /// <exception cref="Autodesk.Forge.Client.ApiException">Thrown when fails to make API call</exception>
        /// <param name="projectId">the &#x60;project id&#x60;</param>
        /// <param name="folderId">the &#x60;folder id&#x60;</param>
        /// <param name="filterType">filter by the &#x60;type&#x60; of the &#x60;ref&#x60; target (optional)</param>
        /// <param name="filterId">filter by the &#x60;id&#x60; of the &#x60;ref&#x60; target (optional)</param>
        /// <param name="filterExtensionType">filter by the extension type (optional)</param>
        /// <returns>ApiResponse of JsonApiCollection</returns>
        public ApiResponse< /*JsonApiCollection*/dynamic > GetFolderRefsWithHttpInfo (string projectId, string folderId, List<string> filterType = null, List<string> filterId = null, List<string> filterExtensionType = null)
        {
            // verify the required parameter 'projectId' is set
            if (projectId == null)
                throw new ApiException(400, "Missing required parameter 'projectId' when calling FoldersApi->GetFolderRefs");
            // verify the required parameter 'folderId' is set
            if (folderId == null)
                throw new ApiException(400, "Missing required parameter 'folderId' when calling FoldersApi->GetFolderRefs");

            var localVarPath = "/data/v1/projects/{project_id}/folders/{folder_id}/refs";
            var localVarPathParams = new Dictionary<String, String>();
            var localVarQueryParams = new Dictionary<String, String>();
            var localVarHeaderParams = new Dictionary<String, String>(Configuration.DefaultHeader);
            var localVarFormParams = new Dictionary<String, String>();
            var localVarFileParams = new Dictionary<String, FileParameter>();
            Object localVarPostBody = null;

            // to determine the Content-Type header
            String[] localVarHttpContentTypes = new String[] {
                "application/vnd.api+json"
            };
            String localVarHttpContentType = Configuration.ApiClient.SelectHeaderContentType(localVarHttpContentTypes);

            // to determine the Accept header
            String[] localVarHttpHeaderAccepts = new String[] {
                "application/vnd.api+json", 
                "application/json"
            };
            String localVarHttpHeaderAccept = Configuration.ApiClient.SelectHeaderAccept(localVarHttpHeaderAccepts);
            if (localVarHttpHeaderAccept != null)
                localVarHeaderParams.Add("Accept", localVarHttpHeaderAccept);

            // set "format" to json by default
            // e.g. /pet/{petId}.{format} becomes /pet/{petId}.json
            localVarPathParams.Add("format", "json");
            if (projectId != null) localVarPathParams.Add("project_id", Configuration.ApiClient.ParameterToString(projectId)); // path parameter
            if (folderId != null) localVarPathParams.Add("folder_id", Configuration.ApiClient.ParameterToString(folderId)); // path parameter
            if (filterType != null) localVarQueryParams.Add("filter[type]", Configuration.ApiClient.ParameterToString(filterType)); // query parameter
            if (filterId != null) localVarQueryParams.Add("filter[id]", Configuration.ApiClient.ParameterToString(filterId)); // query parameter
            if (filterExtensionType != null) localVarQueryParams.Add("filter[extension.type]", Configuration.ApiClient.ParameterToString(filterExtensionType)); // query parameter

            // authentication (oauth2_access_code) required
            // oauth required
            if (!String.IsNullOrEmpty(Configuration.AccessToken))
            {
                localVarHeaderParams["Authorization"] = "Bearer " + Configuration.AccessToken;
            }

            // make the HTTP request
            IRestResponse localVarResponse = (IRestResponse) Configuration.ApiClient.CallApi(localVarPath,
                Method.GET, localVarQueryParams, localVarPostBody, localVarHeaderParams, localVarFormParams, localVarFileParams,
                localVarPathParams, localVarHttpContentType);

            int localVarStatusCode = (int) localVarResponse.StatusCode;

            if (ExceptionFactory != null)
            {
                Exception exception = ExceptionFactory("GetFolderRefs", localVarResponse);
                if (exception != null) throw exception;
            }

            return new ApiResponse</*JsonApiCollection*/dynamic>(localVarStatusCode,
                localVarResponse.Headers.ToDictionary(x => x.Name, x => x.Value.ToString()),
                /*(JsonApiCollection)*/ Configuration.ApiClient.Deserialize(localVarResponse, typeof(JsonApiCollection)));
            
        }

        /// <summary>
        ///  Returns the resources (&#x60;items&#x60;, &#x60;folders&#x60;, and &#x60;versions&#x60;) which have a custom relationship with the given &#x60;folder_id&#x60;. Custom relationships can be established between a folder and other resources within the &#39;data&#39; domain service (folders, items, and versions). 
        /// </summary>
        /// <exception cref="Autodesk.Forge.Client.ApiException">Thrown when fails to make API call</exception>
        /// <param name="projectId">the &#x60;project id&#x60;</param>
        /// <param name="folderId">the &#x60;folder id&#x60;</param>
        /// <param name="filterType">filter by the &#x60;type&#x60; of the &#x60;ref&#x60; target (optional)</param>
        /// <param name="filterId">filter by the &#x60;id&#x60; of the &#x60;ref&#x60; target (optional)</param>
        /// <param name="filterExtensionType">filter by the extension type (optional)</param>
        /// <returns>Task of JsonApiCollection</returns>
        public async System.Threading.Tasks.Task</*JsonApiCollection*/dynamic> GetFolderRefsAsync (string projectId, string folderId, List<string> filterType = null, List<string> filterId = null, List<string> filterExtensionType = null)
        {
             ApiResponse</*JsonApiCollection*/dynamic> localVarResponse = await GetFolderRefsAsyncWithHttpInfo(projectId, folderId, filterType, filterId, filterExtensionType);
             return localVarResponse.Data;
        }

        /// <summary>
        ///  Returns the resources (&#x60;items&#x60;, &#x60;folders&#x60;, and &#x60;versions&#x60;) which have a custom relationship with the given &#x60;folder_id&#x60;. Custom relationships can be established between a folder and other resources within the &#39;data&#39; domain service (folders, items, and versions). 
        /// </summary>
        /// <exception cref="Autodesk.Forge.Client.ApiException">Thrown when fails to make API call</exception>
        /// <param name="projectId">the &#x60;project id&#x60;</param>
        /// <param name="folderId">the &#x60;folder id&#x60;</param>
        /// <param name="filterType">filter by the &#x60;type&#x60; of the &#x60;ref&#x60; target (optional)</param>
        /// <param name="filterId">filter by the &#x60;id&#x60; of the &#x60;ref&#x60; target (optional)</param>
        /// <param name="filterExtensionType">filter by the extension type (optional)</param>
        /// <returns>Task of ApiResponse (JsonApiCollection)</returns>
        public async System.Threading.Tasks.Task<ApiResponse</*JsonApiCollection*/dynamic>> GetFolderRefsAsyncWithHttpInfo (string projectId, string folderId, List<string> filterType = null, List<string> filterId = null, List<string> filterExtensionType = null)
        {
            // verify the required parameter 'projectId' is set
            if (projectId == null)
                throw new ApiException(400, "Missing required parameter 'projectId' when calling FoldersApi->GetFolderRefs");
            // verify the required parameter 'folderId' is set
            if (folderId == null)
                throw new ApiException(400, "Missing required parameter 'folderId' when calling FoldersApi->GetFolderRefs");

            var localVarPath = "/data/v1/projects/{project_id}/folders/{folder_id}/refs";
            var localVarPathParams = new Dictionary<String, String>();
            var localVarQueryParams = new Dictionary<String, String>();
            var localVarHeaderParams = new Dictionary<String, String>(Configuration.DefaultHeader);
            var localVarFormParams = new Dictionary<String, String>();
            var localVarFileParams = new Dictionary<String, FileParameter>();
            Object localVarPostBody = null;

            // to determine the Content-Type header
            String[] localVarHttpContentTypes = new String[] {
                "application/vnd.api+json"
            };
            String localVarHttpContentType = Configuration.ApiClient.SelectHeaderContentType(localVarHttpContentTypes);

            // to determine the Accept header
            String[] localVarHttpHeaderAccepts = new String[] {
                "application/vnd.api+json", 
                "application/json"
            };
            String localVarHttpHeaderAccept = Configuration.ApiClient.SelectHeaderAccept(localVarHttpHeaderAccepts);
            if (localVarHttpHeaderAccept != null)
                localVarHeaderParams.Add("Accept", localVarHttpHeaderAccept);

            // set "format" to json by default
            // e.g. /pet/{petId}.{format} becomes /pet/{petId}.json
            localVarPathParams.Add("format", "json");
            if (projectId != null) localVarPathParams.Add("project_id", Configuration.ApiClient.ParameterToString(projectId)); // path parameter
            if (folderId != null) localVarPathParams.Add("folder_id", Configuration.ApiClient.ParameterToString(folderId)); // path parameter
            if (filterType != null) localVarQueryParams.Add("filter[type]", Configuration.ApiClient.ParameterToString(filterType)); // query parameter
            if (filterId != null) localVarQueryParams.Add("filter[id]", Configuration.ApiClient.ParameterToString(filterId)); // query parameter
            if (filterExtensionType != null) localVarQueryParams.Add("filter[extension.type]", Configuration.ApiClient.ParameterToString(filterExtensionType)); // query parameter

            // authentication (oauth2_access_code) required
            // oauth required
            if (!String.IsNullOrEmpty(Configuration.AccessToken))
            {
                localVarHeaderParams["Authorization"] = "Bearer " + Configuration.AccessToken;
            }

            // make the HTTP request
            IRestResponse localVarResponse = (IRestResponse) await Configuration.ApiClient.CallApiAsync(localVarPath,
                Method.GET, localVarQueryParams, localVarPostBody, localVarHeaderParams, localVarFormParams, localVarFileParams,
                localVarPathParams, localVarHttpContentType);

            int localVarStatusCode = (int) localVarResponse.StatusCode;

            if (ExceptionFactory != null)
            {
                Exception exception = ExceptionFactory("GetFolderRefs", localVarResponse);
                if (exception != null) throw exception;
            }

            return new ApiResponse</*JsonApiCollection*/dynamic>(localVarStatusCode,
                localVarResponse.Headers.ToDictionary(x => x.Name, x => x.Value.ToString()),
                /*(JsonApiCollection)*/ Configuration.ApiClient.Deserialize(localVarResponse, typeof(JsonApiCollection)));
            
        }

        /// <summary>
        ///  Returns the custom relationships that are associated to the given &#x60;folder_id&#x60;. Custom relationships can be established between a folder and other resources within the &#39;data&#39; domain service (folders, items, and versions). 
        /// </summary>
        /// <exception cref="Autodesk.Forge.Client.ApiException">Thrown when fails to make API call</exception>
        /// <param name="projectId">the &#x60;project id&#x60;</param>
        /// <param name="folderId">the &#x60;folder id&#x60;</param>
        /// <param name="filterType">filter by the &#x60;type&#x60; of the &#x60;ref&#x60; target (optional)</param>
        /// <param name="filterId">filter by the &#x60;id&#x60; of the &#x60;ref&#x60; target (optional)</param>
        /// <param name="filterRefType">filter by &#x60;refType&#x60; (optional)</param>
        /// <param name="filterDirection">filter by the direction of the reference (optional)</param>
        /// <param name="filterExtensionType">filter by the extension type (optional)</param>
        /// <returns>Refs</returns>
        public /*Refs*/dynamic GetFolderRelationshipsRefs (string projectId, string folderId, List<string> filterType = null, List<string> filterId = null, List<string> filterRefType = null, string filterDirection = null, List<string> filterExtensionType = null)
        {
             ApiResponse</*Refs*/dynamic> localVarResponse = GetFolderRelationshipsRefsWithHttpInfo(projectId, folderId, filterType, filterId, filterRefType, filterDirection, filterExtensionType);
             return localVarResponse.Data;
        }

        /// <summary>
        ///  Returns the custom relationships that are associated to the given &#x60;folder_id&#x60;. Custom relationships can be established between a folder and other resources within the &#39;data&#39; domain service (folders, items, and versions). 
        /// </summary>
        /// <exception cref="Autodesk.Forge.Client.ApiException">Thrown when fails to make API call</exception>
        /// <param name="projectId">the &#x60;project id&#x60;</param>
        /// <param name="folderId">the &#x60;folder id&#x60;</param>
        /// <param name="filterType">filter by the &#x60;type&#x60; of the &#x60;ref&#x60; target (optional)</param>
        /// <param name="filterId">filter by the &#x60;id&#x60; of the &#x60;ref&#x60; target (optional)</param>
        /// <param name="filterRefType">filter by &#x60;refType&#x60; (optional)</param>
        /// <param name="filterDirection">filter by the direction of the reference (optional)</param>
        /// <param name="filterExtensionType">filter by the extension type (optional)</param>
        /// <returns>ApiResponse of Refs</returns>
        public ApiResponse< /*Refs*/dynamic > GetFolderRelationshipsRefsWithHttpInfo (string projectId, string folderId, List<string> filterType = null, List<string> filterId = null, List<string> filterRefType = null, string filterDirection = null, List<string> filterExtensionType = null)
        {
            // verify the required parameter 'projectId' is set
            if (projectId == null)
                throw new ApiException(400, "Missing required parameter 'projectId' when calling FoldersApi->GetFolderRelationshipsRefs");
            // verify the required parameter 'folderId' is set
            if (folderId == null)
                throw new ApiException(400, "Missing required parameter 'folderId' when calling FoldersApi->GetFolderRelationshipsRefs");

            var localVarPath = "/data/v1/projects/{project_id}/folders/{folder_id}/relationships/refs";
            var localVarPathParams = new Dictionary<String, String>();
            var localVarQueryParams = new Dictionary<String, String>();
            var localVarHeaderParams = new Dictionary<String, String>(Configuration.DefaultHeader);
            var localVarFormParams = new Dictionary<String, String>();
            var localVarFileParams = new Dictionary<String, FileParameter>();
            Object localVarPostBody = null;

            // to determine the Content-Type header
            String[] localVarHttpContentTypes = new String[] {
                "application/vnd.api+json"
            };
            String localVarHttpContentType = Configuration.ApiClient.SelectHeaderContentType(localVarHttpContentTypes);

            // to determine the Accept header
            String[] localVarHttpHeaderAccepts = new String[] {
                "application/vnd.api+json", 
                "application/json"
            };
            String localVarHttpHeaderAccept = Configuration.ApiClient.SelectHeaderAccept(localVarHttpHeaderAccepts);
            if (localVarHttpHeaderAccept != null)
                localVarHeaderParams.Add("Accept", localVarHttpHeaderAccept);

            // set "format" to json by default
            // e.g. /pet/{petId}.{format} becomes /pet/{petId}.json
            localVarPathParams.Add("format", "json");
            if (projectId != null) localVarPathParams.Add("project_id", Configuration.ApiClient.ParameterToString(projectId)); // path parameter
            if (folderId != null) localVarPathParams.Add("folder_id", Configuration.ApiClient.ParameterToString(folderId)); // path parameter
            if (filterType != null) localVarQueryParams.Add("filter[type]", Configuration.ApiClient.ParameterToString(filterType)); // query parameter
            if (filterId != null) localVarQueryParams.Add("filter[id]", Configuration.ApiClient.ParameterToString(filterId)); // query parameter
            if (filterRefType != null) localVarQueryParams.Add("filter[refType]", Configuration.ApiClient.ParameterToString(filterRefType)); // query parameter
            if (filterDirection != null) localVarQueryParams.Add("filter[direction]", Configuration.ApiClient.ParameterToString(filterDirection)); // query parameter
            if (filterExtensionType != null) localVarQueryParams.Add("filter[extension.type]", Configuration.ApiClient.ParameterToString(filterExtensionType)); // query parameter

            // authentication (oauth2_access_code) required
            // oauth required
            if (!String.IsNullOrEmpty(Configuration.AccessToken))
            {
                localVarHeaderParams["Authorization"] = "Bearer " + Configuration.AccessToken;
            }

            // make the HTTP request
            IRestResponse localVarResponse = (IRestResponse) Configuration.ApiClient.CallApi(localVarPath,
                Method.GET, localVarQueryParams, localVarPostBody, localVarHeaderParams, localVarFormParams, localVarFileParams,
                localVarPathParams, localVarHttpContentType);

            int localVarStatusCode = (int) localVarResponse.StatusCode;

            if (ExceptionFactory != null)
            {
                Exception exception = ExceptionFactory("GetFolderRelationshipsRefs", localVarResponse);
                if (exception != null) throw exception;
            }

            return new ApiResponse</*Refs*/dynamic>(localVarStatusCode,
                localVarResponse.Headers.ToDictionary(x => x.Name, x => x.Value.ToString()),
                /*(Refs)*/ Configuration.ApiClient.Deserialize(localVarResponse, typeof(Refs)));
            
        }

        /// <summary>
        ///  Returns the custom relationships that are associated to the given &#x60;folder_id&#x60;. Custom relationships can be established between a folder and other resources within the &#39;data&#39; domain service (folders, items, and versions). 
        /// </summary>
        /// <exception cref="Autodesk.Forge.Client.ApiException">Thrown when fails to make API call</exception>
        /// <param name="projectId">the &#x60;project id&#x60;</param>
        /// <param name="folderId">the &#x60;folder id&#x60;</param>
        /// <param name="filterType">filter by the &#x60;type&#x60; of the &#x60;ref&#x60; target (optional)</param>
        /// <param name="filterId">filter by the &#x60;id&#x60; of the &#x60;ref&#x60; target (optional)</param>
        /// <param name="filterRefType">filter by &#x60;refType&#x60; (optional)</param>
        /// <param name="filterDirection">filter by the direction of the reference (optional)</param>
        /// <param name="filterExtensionType">filter by the extension type (optional)</param>
        /// <returns>Task of Refs</returns>
        public async System.Threading.Tasks.Task</*Refs*/dynamic> GetFolderRelationshipsRefsAsync (string projectId, string folderId, List<string> filterType = null, List<string> filterId = null, List<string> filterRefType = null, string filterDirection = null, List<string> filterExtensionType = null)
        {
             ApiResponse</*Refs*/dynamic> localVarResponse = await GetFolderRelationshipsRefsAsyncWithHttpInfo(projectId, folderId, filterType, filterId, filterRefType, filterDirection, filterExtensionType);
             return localVarResponse.Data;
        }

        /// <summary>
        ///  Returns the custom relationships that are associated to the given &#x60;folder_id&#x60;. Custom relationships can be established between a folder and other resources within the &#39;data&#39; domain service (folders, items, and versions). 
        /// </summary>
        /// <exception cref="Autodesk.Forge.Client.ApiException">Thrown when fails to make API call</exception>
        /// <param name="projectId">the &#x60;project id&#x60;</param>
        /// <param name="folderId">the &#x60;folder id&#x60;</param>
        /// <param name="filterType">filter by the &#x60;type&#x60; of the &#x60;ref&#x60; target (optional)</param>
        /// <param name="filterId">filter by the &#x60;id&#x60; of the &#x60;ref&#x60; target (optional)</param>
        /// <param name="filterRefType">filter by &#x60;refType&#x60; (optional)</param>
        /// <param name="filterDirection">filter by the direction of the reference (optional)</param>
        /// <param name="filterExtensionType">filter by the extension type (optional)</param>
        /// <returns>Task of ApiResponse (Refs)</returns>
        public async System.Threading.Tasks.Task<ApiResponse</*Refs*/dynamic>> GetFolderRelationshipsRefsAsyncWithHttpInfo (string projectId, string folderId, List<string> filterType = null, List<string> filterId = null, List<string> filterRefType = null, string filterDirection = null, List<string> filterExtensionType = null)
        {
            // verify the required parameter 'projectId' is set
            if (projectId == null)
                throw new ApiException(400, "Missing required parameter 'projectId' when calling FoldersApi->GetFolderRelationshipsRefs");
            // verify the required parameter 'folderId' is set
            if (folderId == null)
                throw new ApiException(400, "Missing required parameter 'folderId' when calling FoldersApi->GetFolderRelationshipsRefs");

            var localVarPath = "/data/v1/projects/{project_id}/folders/{folder_id}/relationships/refs";
            var localVarPathParams = new Dictionary<String, String>();
            var localVarQueryParams = new Dictionary<String, String>();
            var localVarHeaderParams = new Dictionary<String, String>(Configuration.DefaultHeader);
            var localVarFormParams = new Dictionary<String, String>();
            var localVarFileParams = new Dictionary<String, FileParameter>();
            Object localVarPostBody = null;

            // to determine the Content-Type header
            String[] localVarHttpContentTypes = new String[] {
                "application/vnd.api+json"
            };
            String localVarHttpContentType = Configuration.ApiClient.SelectHeaderContentType(localVarHttpContentTypes);

            // to determine the Accept header
            String[] localVarHttpHeaderAccepts = new String[] {
                "application/vnd.api+json", 
                "application/json"
            };
            String localVarHttpHeaderAccept = Configuration.ApiClient.SelectHeaderAccept(localVarHttpHeaderAccepts);
            if (localVarHttpHeaderAccept != null)
                localVarHeaderParams.Add("Accept", localVarHttpHeaderAccept);

            // set "format" to json by default
            // e.g. /pet/{petId}.{format} becomes /pet/{petId}.json
            localVarPathParams.Add("format", "json");
            if (projectId != null) localVarPathParams.Add("project_id", Configuration.ApiClient.ParameterToString(projectId)); // path parameter
            if (folderId != null) localVarPathParams.Add("folder_id", Configuration.ApiClient.ParameterToString(folderId)); // path parameter
            if (filterType != null) localVarQueryParams.Add("filter[type]", Configuration.ApiClient.ParameterToString(filterType)); // query parameter
            if (filterId != null) localVarQueryParams.Add("filter[id]", Configuration.ApiClient.ParameterToString(filterId)); // query parameter
            if (filterRefType != null) localVarQueryParams.Add("filter[refType]", Configuration.ApiClient.ParameterToString(filterRefType)); // query parameter
            if (filterDirection != null) localVarQueryParams.Add("filter[direction]", Configuration.ApiClient.ParameterToString(filterDirection)); // query parameter
            if (filterExtensionType != null) localVarQueryParams.Add("filter[extension.type]", Configuration.ApiClient.ParameterToString(filterExtensionType)); // query parameter

            // authentication (oauth2_access_code) required
            // oauth required
            if (!String.IsNullOrEmpty(Configuration.AccessToken))
            {
                localVarHeaderParams["Authorization"] = "Bearer " + Configuration.AccessToken;
            }

            // make the HTTP request
            IRestResponse localVarResponse = (IRestResponse) await Configuration.ApiClient.CallApiAsync(localVarPath,
                Method.GET, localVarQueryParams, localVarPostBody, localVarHeaderParams, localVarFormParams, localVarFileParams,
                localVarPathParams, localVarHttpContentType);

            int localVarStatusCode = (int) localVarResponse.StatusCode;

            if (ExceptionFactory != null)
            {
                Exception exception = ExceptionFactory("GetFolderRelationshipsRefs", localVarResponse);
                if (exception != null) throw exception;
            }

            return new ApiResponse</*Refs*/dynamic>(localVarStatusCode,
                localVarResponse.Headers.ToDictionary(x => x.Name, x => x.Value.ToString()),
                /*(Refs)*/ Configuration.ApiClient.Deserialize(localVarResponse, typeof(Refs)));
            
        }

        public dynamic PatchFolder(string projectId, PatchFolder body)
        {
            return PatchFolderWithHttpInfo(projectId, body).Data;
        }

        public ApiResponse<dynamic> PatchFolderWithHttpInfo(string projectId, PatchFolder body)
        {
            // verify the required parameter 'projectId' is set
            if (projectId == null)
                throw new ApiException(400, "Missing required parameter 'projectId' when calling FoldersApi->PatchFolder");
            // verify the required parameter 'body' is set
            if (body == null)
                throw new ApiException(400, "Missing required parameter 'body' when calling FoldersApi->PatchFolder");

            var localVarPath = "/data/v1/projects/{project_id}/folders/{folder_id}";
            var localVarPathParams = new Dictionary<String, String>();
            var localVarQueryParams = new Dictionary<String, String>();
            var localVarHeaderParams = new Dictionary<String, String>(Configuration.DefaultHeader);
            var localVarFormParams = new Dictionary<String, String>();
            var localVarFileParams = new Dictionary<String, FileParameter>();
            Object localVarPostBody = null;

            // to determine the Content-Type header
            String[] localVarHttpContentTypes = new String[] {
                "application/vnd.api+json"
            };
            String localVarHttpContentType = Configuration.ApiClient.SelectHeaderContentType(localVarHttpContentTypes);

            // to determine the Accept header
            String[] localVarHttpHeaderAccepts = new String[] {
                "application/vnd.api+json",
                "application/json"
            };
            String localVarHttpHeaderAccept = Configuration.ApiClient.SelectHeaderAccept(localVarHttpHeaderAccepts);
            if (localVarHttpHeaderAccept != null)
                localVarHeaderParams.Add("Accept", localVarHttpHeaderAccept);

            // set "format" to json by default
            // e.g. /pet/{petId}.{format} becomes /pet/{petId}.json
            localVarPathParams.Add("format", "json");
            if (projectId != null) localVarPathParams.Add("project_id", Configuration.ApiClient.ParameterToString(projectId)); // path parameter
            if (body.Data != null && body.Data.Id != null) localVarPathParams.Add("folder_id", Configuration.ApiClient.ParameterToString(body.Data.Id)); // path parameter

            if (body != null && body.GetType() == typeof(byte[])) // http body (model) parameter
            {
                localVarPostBody = body; // byte array
            }
            else if (body != null
                     && (body.GetType() == typeof(System.IO.FileStream)
                         || body.GetType() == typeof(System.IO.BinaryReader)
                         || body.GetType() == typeof(System.IO.BufferedStream)
                            || body.GetType() == typeof(System.IO.MemoryStream)
                            || body.GetType() == typeof(System.IO.StreamReader)
                           )
                    )
            {
                localVarPostBody = Configuration.ApiClient.toByteArray(body); // byte array
            }
            else if (body != null)
            {
                localVarPostBody = Configuration.ApiClient.Serialize(body);
            }

            // authentication (oauth2_access_code) required
            // oauth required
            if (!String.IsNullOrEmpty(Configuration.AccessToken))
            {
                localVarHeaderParams["Authorization"] = "Bearer " + Configuration.AccessToken;
            }

            // make the HTTP request
            IRestResponse localVarResponse = (IRestResponse)Configuration.ApiClient.CallApi(localVarPath,
                Method.PATCH, localVarQueryParams, localVarPostBody, localVarHeaderParams, localVarFormParams, localVarFileParams,
                localVarPathParams, localVarHttpContentType);

            int localVarStatusCode = (int)localVarResponse.StatusCode;

            if (ExceptionFactory != null)
            {
                Exception exception = ExceptionFactory("PatchFolder", localVarResponse);
                if (exception != null) throw exception;
            }


            return new ApiResponse</*Folder*/dynamic>(localVarStatusCode,
                localVarResponse.Headers.ToDictionary(x => x.Name, x => x.Value.ToString()),
                /*(Folder)*/ Configuration.ApiClient.Deserialize(localVarResponse, typeof(Folder)));
        }

        /// <summary>
        ///  Deletes or renames a folder in the &#x60;data&#x60; domain service
        /// </summary>
        /// <exception cref="Autodesk.Forge.Client.ApiException">Thrown when fails to make API call</exception>
        /// <param name="projectId">the &#x60;project id&#x60;</param>
        /// <param name="body">describe the folder to be updated</param>
        /// <returns>Task of DynamicJsonResponse</returns>
        public async System.Threading.Tasks.Task<dynamic> PatchFolderAsync(string projectId, PatchFolder body)
        {
            ApiResponse<dynamic> localVarResponse = await PatchFolderAsyncWithHttpInfo(projectId, body);
            return localVarResponse.Data;
        }

        /// <summary>
        ///  Deletes or renames a folder in the &#x60;data&#x60; domain service
        /// </summary>
        /// <exception cref="Autodesk.Forge.Client.ApiException">Thrown when fails to make API call</exception>
        /// <param name="projectId">the &#x60;project id&#x60;</param>
        /// <param name="body">describe the folder to be updated</param>
        /// <returns>Task of ApiResponse</returns>
        public async System.Threading.Tasks.Task<ApiResponse<dynamic>> PatchFolderAsyncWithHttpInfo(string projectId, PatchFolder body)
        {
            // verify the required parameter 'projectId' is set
            if (projectId == null)
                throw new ApiException(400, "Missing required parameter 'projectId' when calling FoldersApi->PatchFolder");
            // verify the required parameter 'folderId' is set
            if (body == null)
                throw new ApiException(400, "Missing required parameter 'body' when calling FoldersApi->PatchFolder");

            var localVarPath = "/data/v1/projects/{project_id}/folders/{folder_id}";
            var localVarPathParams = new Dictionary<String, String>();
            var localVarQueryParams = new Dictionary<String, String>();
            var localVarHeaderParams = new Dictionary<String, String>(Configuration.DefaultHeader);
            var localVarFormParams = new Dictionary<String, String>();
            var localVarFileParams = new Dictionary<String, FileParameter>();
            Object localVarPostBody = null;

            // to determine the Content-Type header
            String[] localVarHttpContentTypes = new String[] {
                "application/vnd.api+json"
            };
            String localVarHttpContentType = Configuration.ApiClient.SelectHeaderContentType(localVarHttpContentTypes);

            // to determine the Accept header
            String[] localVarHttpHeaderAccepts = new String[] {
                "application/vnd.api+json",
                "application/json"
            };
            String localVarHttpHeaderAccept = Configuration.ApiClient.SelectHeaderAccept(localVarHttpHeaderAccepts);
            if (localVarHttpHeaderAccept != null)
                localVarHeaderParams.Add("Accept", localVarHttpHeaderAccept);

            // set "format" to json by default
            // e.g. /pet/{petId}.{format} becomes /pet/{petId}.json
            localVarPathParams.Add("format", "json");
            if (projectId != null) localVarPathParams.Add("project_id", Configuration.ApiClient.ParameterToString(projectId)); // path parameter
            if (body.Data != null && body.Data.Id != null) localVarPathParams.Add("folder_id", Configuration.ApiClient.ParameterToString(body.Data.Id)); // path parameter
            if (body != null && body.GetType() == typeof(byte[])) // http body (model) parameter
            {
                localVarPostBody = body; // byte array
            }
            else if (body != null
                     && (body.GetType() == typeof(System.IO.FileStream)
                         || body.GetType() == typeof(System.IO.BinaryReader)
                         || body.GetType() == typeof(System.IO.BufferedStream)
                            || body.GetType() == typeof(System.IO.MemoryStream)
                            || body.GetType() == typeof(System.IO.StreamReader)
                           )
                    )
            {
                localVarPostBody = Configuration.ApiClient.toByteArray(body); // byte array
            }
            else if (body != null)
            {
                localVarPostBody = Configuration.ApiClient.Serialize(body);
            }



            // authentication (oauth2_access_code) required
            // oauth required
            if (!String.IsNullOrEmpty(Configuration.AccessToken))
            {
                localVarHeaderParams["Authorization"] = "Bearer " + Configuration.AccessToken;
            }

            // make the HTTP request
            IRestResponse localVarResponse = (IRestResponse)await Configuration.ApiClient.CallApiAsync(localVarPath,
                Method.PATCH, localVarQueryParams, localVarPostBody, localVarHeaderParams, localVarFormParams, localVarFileParams,
                localVarPathParams, localVarHttpContentType);

            int localVarStatusCode = (int)localVarResponse.StatusCode;

            if (ExceptionFactory != null)
            {
                Exception exception = ExceptionFactory("PatchFolder", localVarResponse);
                if (exception != null) throw exception;
            }

            return new ApiResponse</*JsonApiCollection*/dynamic>(localVarStatusCode,
                localVarResponse.Headers.ToDictionary(x => x.Name, x => x.Value.ToString()),
                /*(JsonApiCollection)*/ Configuration.ApiClient.Deserialize(localVarResponse, typeof(JsonApiCollection)));


        }

        /// <summary>
        ///  Creates a new folder in the &#x60;data&#x60; domain service
        /// </summary>
        /// <exception cref="Autodesk.Forge.Client.ApiException">Thrown when fails to make API call</exception>
        /// <param name="projectId">the &#x60;project id&#x60;</param>
        /// <param name="body">describe the folder to be created</param>
        /// <returns>ApiResponse of Object</returns>
        public dynamic PostFolder (string projectId, CreateFolder body)
        {
            ApiResponse<dynamic> localVarResponse = PostFolderWithHttpInfo (projectId, body);
            return localVarResponse.Data;
        }

        /// <summary>
        ///  Creates a new folder in the &#x60;data&#x60; domain service
        /// </summary>
        /// <exception cref="Autodesk.Forge.Client.ApiException">Thrown when fails to make API call</exception>
        /// <param name="projectId">the &#x60;project id&#x60;</param>
        /// <param name="body">describe the folder to be created</param>
        /// <returns>ApiResponse of Object</returns>
        public ApiResponse<dynamic> PostFolderWithHttpInfo (string projectId, CreateFolder body)
        {
            // verify the required parameter 'projectId' is set
            if (projectId == null)
                throw new ApiException(400, "Missing required parameter 'projectId' when calling FoldersApi->PostFolder");
            // verify the required parameter 'body' is set
            if (body == null)
                throw new ApiException(400, "Missing required parameter 'body' when calling FoldersApi->PostFolder");

            var localVarPath = "/data/v1/projects/{project_id}/folders";
            var localVarPathParams = new Dictionary<String, String>();
            var localVarQueryParams = new Dictionary<String, String>();
            var localVarHeaderParams = new Dictionary<String, String>(Configuration.DefaultHeader);
            var localVarFormParams = new Dictionary<String, String>();
            var localVarFileParams = new Dictionary<String, FileParameter>();
            Object localVarPostBody = null;

            // to determine the Content-Type header
            String[] localVarHttpContentTypes = new String[] {
                "application/vnd.api+json"
            };
            String localVarHttpContentType = Configuration.ApiClient.SelectHeaderContentType(localVarHttpContentTypes);

            // to determine the Accept header
            String[] localVarHttpHeaderAccepts = new String[] {
                "application/vnd.api+json", 
                "application/json"
            };
            String localVarHttpHeaderAccept = Configuration.ApiClient.SelectHeaderAccept(localVarHttpHeaderAccepts);
            if (localVarHttpHeaderAccept != null)
                localVarHeaderParams.Add("Accept", localVarHttpHeaderAccept);

            // set "format" to json by default
            // e.g. /pet/{petId}.{format} becomes /pet/{petId}.json
            localVarPathParams.Add("format", "json");
            if (projectId != null) localVarPathParams.Add("project_id", Configuration.ApiClient.ParameterToString(projectId)); // path parameter
            if (body != null && body.GetType() == typeof(byte[])) // http body (model) parameter
            {
                localVarPostBody = body; // byte array
            }
            else if (   body != null
                     && (   body.GetType() == typeof(System.IO.FileStream)
                         || body.GetType() == typeof(System.IO.BinaryReader)
                         || body.GetType() == typeof(System.IO.BufferedStream)
                   	     || body.GetType() == typeof(System.IO.MemoryStream)
                   	     || body.GetType() == typeof(System.IO.StreamReader)
                   	    )
                    )
            {
                localVarPostBody =Configuration.ApiClient.toByteArray(body); // byte array
            }
            else if ( body != null )
            {
                localVarPostBody = Configuration.ApiClient.Serialize(body);
            }

            // authentication (oauth2_access_code) required
            // oauth required
            if (!String.IsNullOrEmpty(Configuration.AccessToken))
            {
                localVarHeaderParams["Authorization"] = "Bearer " + Configuration.AccessToken;
            }

            // make the HTTP request
            IRestResponse localVarResponse = (IRestResponse) Configuration.ApiClient.CallApi(localVarPath,
                Method.POST, localVarQueryParams, localVarPostBody, localVarHeaderParams, localVarFormParams, localVarFileParams,
                localVarPathParams, localVarHttpContentType);

            int localVarStatusCode = (int) localVarResponse.StatusCode;

            if (ExceptionFactory != null)
            {
                Exception exception = ExceptionFactory("PostFolder", localVarResponse);
                if (exception != null) throw exception;
            }

            
            return new ApiResponse<Object>(localVarStatusCode,
                localVarResponse.Headers.ToDictionary(x => x.Name, x => x.Value.ToString()),
                /*(JsonApiCollection)*/ Configuration.ApiClient.Deserialize(localVarResponse, typeof(JsonApiCollection)));
        }

        /// <summary>
        ///  Creates a new folder in the &#x60;data&#x60; domain service
        /// </summary>
        /// <exception cref="Autodesk.Forge.Client.ApiException">Thrown when fails to make API call</exception>
        /// <param name="projectId">the &#x60;project id&#x60;</param>
        /// <param name="body">describe the folder to be created</param>
        /// <returns>Task of void</returns>
        public async System.Threading.Tasks.Task<dynamic> PostFolderAsync (string projectId, CreateFolder body)
        {
            ApiResponse<dynamic> localVarResponse = await PostFolderAsyncWithHttpInfo(projectId, body);
            return localVarResponse.Data;
        }

        /// <summary>
        ///  Creates a new folder in the &#x60;data&#x60; domain service
        /// </summary>
        /// <exception cref="Autodesk.Forge.Client.ApiException">Thrown when fails to make API call</exception>
        /// <param name="projectId">the &#x60;project id&#x60;</param>
        /// <param name="body">describe the folder to be created</param>
        /// <returns>Task of ApiResponse</returns>
        public async System.Threading.Tasks.Task<ApiResponse<dynamic>> PostFolderAsyncWithHttpInfo (string projectId, CreateFolder body)
        {
            // verify the required parameter 'projectId' is set
            if (projectId == null)
                throw new ApiException(400, "Missing required parameter 'projectId' when calling FoldersApi->PostFolder");
            // verify the required parameter 'body' is set
            if (body == null)
                throw new ApiException(400, "Missing required parameter 'body' when calling FoldersApi->PostFolder");

            var localVarPath = "/data/v1/projects/{project_id}/folders";
            var localVarPathParams = new Dictionary<String, String>();
            var localVarQueryParams = new Dictionary<String, String>();
            var localVarHeaderParams = new Dictionary<String, String>(Configuration.DefaultHeader);
            var localVarFormParams = new Dictionary<String, String>();
            var localVarFileParams = new Dictionary<String, FileParameter>();
            Object localVarPostBody = null;

            // to determine the Content-Type header
            String[] localVarHttpContentTypes = new String[] {
                "application/vnd.api+json"
            };
            String localVarHttpContentType = Configuration.ApiClient.SelectHeaderContentType(localVarHttpContentTypes);

            // to determine the Accept header
            String[] localVarHttpHeaderAccepts = new String[] {
                "application/vnd.api+json", 
                "application/json"
            };
            String localVarHttpHeaderAccept = Configuration.ApiClient.SelectHeaderAccept(localVarHttpHeaderAccepts);
            if (localVarHttpHeaderAccept != null)
                localVarHeaderParams.Add("Accept", localVarHttpHeaderAccept);

            // set "format" to json by default
            // e.g. /pet/{petId}.{format} becomes /pet/{petId}.json
            localVarPathParams.Add("format", "json");
            if (projectId != null) localVarPathParams.Add("project_id", Configuration.ApiClient.ParameterToString(projectId)); // path parameter
            if (body != null && body.GetType() == typeof(byte[])) // http body (model) parameter
            {
                localVarPostBody = body; // byte array
            }
            else if (   body != null
                     && (   body.GetType() == typeof(System.IO.FileStream)
                         || body.GetType() == typeof(System.IO.BinaryReader)
                         || body.GetType() == typeof(System.IO.BufferedStream)
                   	     || body.GetType() == typeof(System.IO.MemoryStream)
                   	     || body.GetType() == typeof(System.IO.StreamReader)
                   	    )
                    )
            {
                localVarPostBody =Configuration.ApiClient.toByteArray(body); // byte array
            }
            else if ( body != null )
            {
                localVarPostBody = Configuration.ApiClient.Serialize(body);
            }

            // authentication (oauth2_access_code) required
            // oauth required
            if (!String.IsNullOrEmpty(Configuration.AccessToken))
            {
                localVarHeaderParams["Authorization"] = "Bearer " + Configuration.AccessToken;
            }

            // make the HTTP request
            IRestResponse localVarResponse = (IRestResponse) await Configuration.ApiClient.CallApiAsync(localVarPath,
                Method.POST, localVarQueryParams, localVarPostBody, localVarHeaderParams, localVarFormParams, localVarFileParams,
                localVarPathParams, localVarHttpContentType);

            int localVarStatusCode = (int) localVarResponse.StatusCode;

            if (ExceptionFactory != null)
            {
                Exception exception = ExceptionFactory("PostFolder", localVarResponse);
                if (exception != null) throw exception;
            }

            
            return new ApiResponse<Object>(localVarStatusCode,
                localVarResponse.Headers.ToDictionary(x => x.Name, x => x.Value.ToString()),
                 /*(JsonApiCollection)*/ Configuration.ApiClient.Deserialize(localVarResponse, typeof(JsonApiCollection)));
        }

        /// <summary>
        ///  Creates a custom relationship between a folder and another resource within the &#39;data&#39; domain service (folder, item, or version). 
        /// </summary>
        /// <exception cref="Autodesk.Forge.Client.ApiException">Thrown when fails to make API call</exception>
        /// <param name="projectId">the &#x60;project id&#x60;</param>
        /// <param name="folderId">the &#x60;folder id&#x60;</param>
        /// <param name="body">describe the ref to be created</param>
        /// <returns></returns>
        public dynamic PostFolderRelationshipsRef (string projectId, string folderId, CreateRef body)
        {
            ApiResponse<dynamic> localVarResponse = PostFolderRelationshipsRefWithHttpInfo (projectId, folderId, body);
            return localVarResponse.Data;
        }

        /// <summary>
        ///  Creates a custom relationship between a folder and another resource within the &#39;data&#39; domain service (folder, item, or version). 
        /// </summary>
        /// <exception cref="Autodesk.Forge.Client.ApiException">Thrown when fails to make API call</exception>
        /// <param name="projectId">the &#x60;project id&#x60;</param>
        /// <param name="folderId">the &#x60;folder id&#x60;</param>
        /// <param name="body">describe the ref to be created</param>
        /// <returns>ApiResponse of Object(void)</returns>
        public ApiResponse<dynamic> PostFolderRelationshipsRefWithHttpInfo (string projectId, string folderId, CreateRef body)
        {
            // verify the required parameter 'projectId' is set
            if (projectId == null)
                throw new ApiException(400, "Missing required parameter 'projectId' when calling FoldersApi->PostFolderRelationshipsRef");
            // verify the required parameter 'folderId' is set
            if (folderId == null)
                throw new ApiException(400, "Missing required parameter 'folderId' when calling FoldersApi->PostFolderRelationshipsRef");
            // verify the required parameter 'body' is set
            if (body == null)
                throw new ApiException(400, "Missing required parameter 'body' when calling FoldersApi->PostFolderRelationshipsRef");

            var localVarPath = "/data/v1/projects/{project_id}/folders/{folder_id}/relationships/refs";
            var localVarPathParams = new Dictionary<String, String>();
            var localVarQueryParams = new Dictionary<String, String>();
            var localVarHeaderParams = new Dictionary<String, String>(Configuration.DefaultHeader);
            var localVarFormParams = new Dictionary<String, String>();
            var localVarFileParams = new Dictionary<String, FileParameter>();
            Object localVarPostBody = null;

            // to determine the Content-Type header
            String[] localVarHttpContentTypes = new String[] {
                "application/vnd.api+json"
            };
            String localVarHttpContentType = Configuration.ApiClient.SelectHeaderContentType(localVarHttpContentTypes);

            // to determine the Accept header
            String[] localVarHttpHeaderAccepts = new String[] {
                "application/vnd.api+json", 
                "application/json"
            };
            String localVarHttpHeaderAccept = Configuration.ApiClient.SelectHeaderAccept(localVarHttpHeaderAccepts);
            if (localVarHttpHeaderAccept != null)
                localVarHeaderParams.Add("Accept", localVarHttpHeaderAccept);

            // set "format" to json by default
            // e.g. /pet/{petId}.{format} becomes /pet/{petId}.json
            localVarPathParams.Add("format", "json");
            if (projectId != null) localVarPathParams.Add("project_id", Configuration.ApiClient.ParameterToString(projectId)); // path parameter
            if (folderId != null) localVarPathParams.Add("folder_id", Configuration.ApiClient.ParameterToString(folderId)); // path parameter
            if (body != null && body.GetType() == typeof(byte[])) // http body (model) parameter
            {
                localVarPostBody = body; // byte array
            }
            else if (   body != null
                     && (   body.GetType() == typeof(System.IO.FileStream)
                         || body.GetType() == typeof(System.IO.BinaryReader)
                         || body.GetType() == typeof(System.IO.BufferedStream)
                   	     || body.GetType() == typeof(System.IO.MemoryStream)
                   	     || body.GetType() == typeof(System.IO.StreamReader)
                   	    )
                    )
            {
                localVarPostBody =Configuration.ApiClient.toByteArray(body); // byte array
            }
            else if ( body != null )
            {
                localVarPostBody = Configuration.ApiClient.Serialize(body);
            }

            // authentication (oauth2_access_code) required
            // oauth required
            if (!String.IsNullOrEmpty(Configuration.AccessToken))
            {
                localVarHeaderParams["Authorization"] = "Bearer " + Configuration.AccessToken;
            }

            // make the HTTP request
            IRestResponse localVarResponse = (IRestResponse) Configuration.ApiClient.CallApi(localVarPath,
                Method.POST, localVarQueryParams, localVarPostBody, localVarHeaderParams, localVarFormParams, localVarFileParams,
                localVarPathParams, localVarHttpContentType);

            int localVarStatusCode = (int) localVarResponse.StatusCode;

            if (ExceptionFactory != null)
            {
                Exception exception = ExceptionFactory("PostFolderRelationshipsRef", localVarResponse);
                if (exception != null) throw exception;
            }

            
            return new ApiResponse<Object>(localVarStatusCode,
                localVarResponse.Headers.ToDictionary(x => x.Name, x => x.Value.ToString()),
                null);
        }

        /// <summary>
        ///  Creates a custom relationship between a folder and another resource within the &#39;data&#39; domain service (folder, item, or version). 
        /// </summary>
        /// <exception cref="Autodesk.Forge.Client.ApiException">Thrown when fails to make API call</exception>
        /// <param name="projectId">the &#x60;project id&#x60;</param>
        /// <param name="folderId">the &#x60;folder id&#x60;</param>
        /// <param name="body">describe the ref to be created</param>
        /// <returns>Task of void</returns>
        public async System.Threading.Tasks.Task<dynamic> PostFolderRelationshipsRefAsync (string projectId, string folderId, CreateRef body)
        {
            ApiResponse<dynamic> localVarResponse = await PostFolderRelationshipsRefAsyncWithHttpInfo(projectId, folderId, body);
            return localVarResponse.Data;
        }

        /// <summary>
        ///  Creates a custom relationship between a folder and another resource within the &#39;data&#39; domain service (folder, item, or version). 
        /// </summary>
        /// <exception cref="Autodesk.Forge.Client.ApiException">Thrown when fails to make API call</exception>
        /// <param name="projectId">the &#x60;project id&#x60;</param>
        /// <param name="folderId">the &#x60;folder id&#x60;</param>
        /// <param name="body">describe the ref to be created</param>
        /// <returns>Task of ApiResponse</returns>
        public async System.Threading.Tasks.Task<ApiResponse<dynamic>> PostFolderRelationshipsRefAsyncWithHttpInfo (string projectId, string folderId, CreateRef body)
        {
            // verify the required parameter 'projectId' is set
            if (projectId == null)
                throw new ApiException(400, "Missing required parameter 'projectId' when calling FoldersApi->PostFolderRelationshipsRef");
            // verify the required parameter 'folderId' is set
            if (folderId == null)
                throw new ApiException(400, "Missing required parameter 'folderId' when calling FoldersApi->PostFolderRelationshipsRef");
            // verify the required parameter 'body' is set
            if (body == null)
                throw new ApiException(400, "Missing required parameter 'body' when calling FoldersApi->PostFolderRelationshipsRef");

            var localVarPath = "/data/v1/projects/{project_id}/folders/{folder_id}/relationships/refs";
            var localVarPathParams = new Dictionary<String, String>();
            var localVarQueryParams = new Dictionary<String, String>();
            var localVarHeaderParams = new Dictionary<String, String>(Configuration.DefaultHeader);
            var localVarFormParams = new Dictionary<String, String>();
            var localVarFileParams = new Dictionary<String, FileParameter>();
            Object localVarPostBody = null;

            // to determine the Content-Type header
            String[] localVarHttpContentTypes = new String[] {
                "application/vnd.api+json"
            };
            String localVarHttpContentType = Configuration.ApiClient.SelectHeaderContentType(localVarHttpContentTypes);

            // to determine the Accept header
            String[] localVarHttpHeaderAccepts = new String[] {
                "application/vnd.api+json", 
                "application/json"
            };
            String localVarHttpHeaderAccept = Configuration.ApiClient.SelectHeaderAccept(localVarHttpHeaderAccepts);
            if (localVarHttpHeaderAccept != null)
                localVarHeaderParams.Add("Accept", localVarHttpHeaderAccept);

            // set "format" to json by default
            // e.g. /pet/{petId}.{format} becomes /pet/{petId}.json
            localVarPathParams.Add("format", "json");
            if (projectId != null) localVarPathParams.Add("project_id", Configuration.ApiClient.ParameterToString(projectId)); // path parameter
            if (folderId != null) localVarPathParams.Add("folder_id", Configuration.ApiClient.ParameterToString(folderId)); // path parameter
            if (body != null && body.GetType() == typeof(byte[])) // http body (model) parameter
            {
                localVarPostBody = body; // byte array
            }
            else if (   body != null
                     && (   body.GetType() == typeof(System.IO.FileStream)
                         || body.GetType() == typeof(System.IO.BinaryReader)
                         || body.GetType() == typeof(System.IO.BufferedStream)
                   	     || body.GetType() == typeof(System.IO.MemoryStream)
                   	     || body.GetType() == typeof(System.IO.StreamReader)
                   	    )
                    )
            {
                localVarPostBody =Configuration.ApiClient.toByteArray(body); // byte array
            }
            else if ( body != null )
            {
                localVarPostBody = Configuration.ApiClient.Serialize(body);
            }

            // authentication (oauth2_access_code) required
            // oauth required
            if (!String.IsNullOrEmpty(Configuration.AccessToken))
            {
                localVarHeaderParams["Authorization"] = "Bearer " + Configuration.AccessToken;
            }

            // make the HTTP request
            IRestResponse localVarResponse = (IRestResponse) await Configuration.ApiClient.CallApiAsync(localVarPath,
                Method.POST, localVarQueryParams, localVarPostBody, localVarHeaderParams, localVarFormParams, localVarFileParams,
                localVarPathParams, localVarHttpContentType);

            int localVarStatusCode = (int) localVarResponse.StatusCode;

            if (ExceptionFactory != null)
            {
                Exception exception = ExceptionFactory("PostFolderRelationshipsRef", localVarResponse);
                if (exception != null) throw exception;
            }

            
            return new ApiResponse<Object>(localVarStatusCode,
                localVarResponse.Headers.ToDictionary(x => x.Name, x => x.Value.ToString()),
                null);
        }

    }
}
