/* 
 * Forge SDK
 *
 * The Forge Platform contains an expanding collection of web service components that can be used with Autodesk cloud-based products or your own technologies. Take advantage of Autodesk’s expertise in design and engineering.
 *
 * OpenAPI spec version: 0.1.0
 * Contact: forge.help@autodesk.com
 * Generated by: https://github.com/swagger-api/swagger-codegen.git
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

using System;
using System.Linq;
using System.IO;
using System.Text;
using System.Collections;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.Runtime.Serialization;
using Newtonsoft.Json;
using Newtonsoft.Json.Converters;

namespace Autodesk.Forge.Model
{
    /// <summary>
    /// metadata object definition
    /// </summary>
    [DataContract]
    public partial class MetadataObject :  IEquatable<MetadataObject>
    {
        /// <summary>
        /// Initializes a new instance of the <see cref="MetadataObject" /> class.
        /// </summary>
        [JsonConstructorAttribute]
        protected MetadataObject() { }
        /// <summary>
        /// Initializes a new instance of the <see cref="MetadataObject" /> class.
        /// </summary>
        /// <param name="Objectid">Unique ID for the object (required).</param>
        /// <param name="Name">Name of the object (required).</param>
        /// <param name="Objects">Optional collection of “children” objects within the hierarchy.</param>
        public MetadataObject(int? Objectid = null, string Name = null, List<MetadataObject> Objects = null)
        {
            // to ensure "Objectid" is required (not null)
            if (Objectid == null)
            {
                throw new InvalidDataException("Objectid is a required property for MetadataObject and cannot be null");
            }
            else
            {
                this.Objectid = Objectid;
            }
            // to ensure "Name" is required (not null)
            if (Name == null)
            {
                throw new InvalidDataException("Name is a required property for MetadataObject and cannot be null");
            }
            else
            {
                this.Name = Name;
            }
            this.Objects = Objects;
        }
        
        /// <summary>
        /// Unique ID for the object
        /// </summary>
        /// <value>Unique ID for the object</value>
        [DataMember(Name="objectid", EmitDefaultValue=false)]
        public int? Objectid { get; set; }
        /// <summary>
        /// Name of the object
        /// </summary>
        /// <value>Name of the object</value>
        [DataMember(Name="name", EmitDefaultValue=false)]
        public string Name { get; set; }
        /// <summary>
        /// Optional collection of “children” objects within the hierarchy
        /// </summary>
        /// <value>Optional collection of “children” objects within the hierarchy</value>
        [DataMember(Name="objects", EmitDefaultValue=false)]
        public List<MetadataObject> Objects { get; set; }
        /// <summary>
        /// Returns the string presentation of the object
        /// </summary>
        /// <returns>String presentation of the object</returns>
        public override string ToString()
        {
            var sb = new StringBuilder();
            sb.Append("class MetadataObject {\n");
            sb.Append("  Objectid: ").Append(Objectid).Append("\n");
            sb.Append("  Name: ").Append(Name).Append("\n");
            sb.Append("  Objects: ").Append(Objects).Append("\n");
            sb.Append("}\n");
            return sb.ToString();
        }
  
        /// <summary>
        /// Returns the JSON string presentation of the object
        /// </summary>
        /// <returns>JSON string presentation of the object</returns>
        public string ToJson()
        {
            return JsonConvert.SerializeObject(this, Formatting.Indented);
        }

        /// <summary>
        /// Returns true if objects are equal
        /// </summary>
        /// <param name="obj">Object to be compared</param>
        /// <returns>Boolean</returns>
        public override bool Equals(object obj)
        {
            // credit: http://stackoverflow.com/a/10454552/677735
            return this.Equals(obj as MetadataObject);
        }

        /// <summary>
        /// Returns true if MetadataObject instances are equal
        /// </summary>
        /// <param name="other">Instance of MetadataObject to be compared</param>
        /// <returns>Boolean</returns>
        public bool Equals(MetadataObject other)
        {
            // credit: http://stackoverflow.com/a/10454552/677735
            if (other == null)
                return false;

            return 
                (
                    this.Objectid == other.Objectid ||
                    this.Objectid != null &&
                    this.Objectid.Equals(other.Objectid)
                ) && 
                (
                    this.Name == other.Name ||
                    this.Name != null &&
                    this.Name.Equals(other.Name)
                ) && 
                (
                    this.Objects == other.Objects ||
                    this.Objects != null &&
                    this.Objects.SequenceEqual(other.Objects)
                );
        }

        /// <summary>
        /// Gets the hash code
        /// </summary>
        /// <returns>Hash code</returns>
        public override int GetHashCode()
        {
            // credit: http://stackoverflow.com/a/263416/677735
            unchecked // Overflow is fine, just wrap
            {
                int hash = 41;
                // Suitable nullity checks etc, of course :)
                if (this.Objectid != null)
                    hash = hash * 59 + this.Objectid.GetHashCode();
                if (this.Name != null)
                    hash = hash * 59 + this.Name.GetHashCode();
                if (this.Objects != null)
                    hash = hash * 59 + this.Objects.GetHashCode();
                return hash;
            }
        }
    }

}

